@@ -53,7 +53,6 @@
 #include <kapplication.h>
 #include <kstandarddirs.h>
 #include <kiconloader.h>
-#include <kaudioplayer.h>
 #include <kurl.h>
 #include <kurldrag.h>
 #include <klocale.h>
@@ -66,6 +65,9 @@
 #include <kstatusbar.h>
 #include <kconfig.h>
 #include <kio/netaccess.h>
+#include <knotifyclient.h>
+#include <knotifydialog.h>
+#include <kmenubar.h>
 
 #include "safedelete.h"
 #include "settings.h"
@@ -82,6 +84,7 @@
 
 
 #include <kio/authinfo.h>
+#include <kio/global.h>
 #include <qiconset.h>
 
 #include "version.h"
@@ -148,6 +151,8 @@ KMainWidget::KMainWidget(bool bStartDocked)
     // Setup log window
     logWindow = new LogWindow();
 
+    m_showDropTarget = false;
+
     setCaption(KGETVERSION);
 
     setupGUI();
@@ -207,7 +212,7 @@ KMainWidget::KMainWidget(bool bStartDocked)
     readTransfers();
 
     // Setup special windows
-    kdrop = new DropTarget();
+    kdrop = new DropTarget(this);
     kdock = new DockWidget(this);
     connect(kdock, SIGNAL(quitSelected()), SLOT(slotQuit()));
 
@@ -222,7 +227,6 @@ KMainWidget::KMainWidget(bool bStartDocked)
 
     // update actions
     m_paUseAnimation->setChecked(ksettings.b_useAnimation);
-    m_paUseSound->setChecked(ksettings.b_useSound);
     m_paExpertMode->setChecked(ksettings.b_expertMode);
     m_paUseLastDir->setChecked(ksettings.b_useLastDir);
 
@@ -252,6 +256,10 @@ KMainWidget::KMainWidget(bool bStartDocked)
 
     kdock->show();
 
+    KNotifyClient::startDaemon();
+
+    setStandardToolBarMenuEnabled(true);
+
 #ifdef _DEBUG
     sDebugOut << endl;
 #endif
@@ -350,7 +358,7 @@ void KMainWidget::setupGUI()
 #endif
     m_paResume = new KAction(i18n("&Resume"),"tool_resume", 0, this, SLOT(slotResumeCurrent()), coll, "resume");
     m_paPause = new KAction(i18n("&Pause"),"tool_pause", 0, this, SLOT(slotPauseCurrent()), coll, "pause");
-    m_paDelete = new KAction(i18n("&Delete"),"tool_delete", Qt::Key_Delete, this, SLOT(slotDeleteCurrent()), coll, "delete");
+    m_paDelete = new KAction(i18n("&Delete"),"editdelete", Qt::Key_Delete, this, SLOT(slotDeleteCurrent()), coll, "delete");
     m_paRestart = new KAction(i18n("Re&start"),"tool_restart", 0, this, SLOT(slotRestartCurrent()), coll, "restart");
 
     m_paQueue = new KRadioAction(i18n("&Queue"),"tool_queue", 0, this, SLOT(slotQueueCurrent()), coll, "queue");
@@ -363,7 +371,6 @@ void KMainWidget::setupGUI()
 
     // options actions
     m_paUseAnimation   =  new KToggleAction(i18n("Use &Animation"), 0, this, SLOT(slotToggleAnimation()), coll, "toggle_animation");
-    m_paUseSound       =  new KToggleAction(i18n("Use &Sound"), 0, this, SLOT(slotToggleSound()), coll, "toggle_sound");
     m_paExpertMode     =  new KToggleAction(i18n("&Expert Mode"),"tool_expert", 0, this, SLOT(slotToggleExpertMode()), coll, "expert_mode");
     m_paUseLastDir     =  new KToggleAction(i18n("&Use-Last-Folder Mode"),"tool_uselastdir", 0, this, SLOT(slotToggleUseLastDir()), coll, "use_last_dir");
     m_paAutoDisconnect =  new KToggleAction(i18n("Auto-&Disconnect Mode"),"tool_disconnect", 0, this, SLOT(slotToggleAutoDisconnect()), coll, "auto_disconnect");
@@ -373,16 +380,23 @@ void KMainWidget::setupGUI()
 
     m_paPreferences    =  KStdAction::preferences(this, SLOT(slotPreferences()), coll);
 
-    KStdAction::keyBindings(guiFactory(), SLOT(configureShortcuts()),
-actionCollection());
+    KStdAction::keyBindings(guiFactory(), SLOT(configureShortcuts()), coll);
     KStdAction::configureToolbars(this, SLOT(slotConfigureToolbars()), coll);
+    KStdAction::configureNotifications(this, SLOT(slotConfigureNotifications()), coll);
+
+    m_menubarAction = KStdAction::showMenubar(this, SLOT(slotShowMenubar()), coll, "settings_showmenubar" );
+    m_menubarAction->setChecked( !menuBar()->isHidden() );
 
     // view actions
     createStandardStatusBarAction();
 
     m_paShowLog      = new KToggleAction(i18n("Show &Log Window"),"tool_logwindow", 0, this, SLOT(slotToggleLogWindow()), coll, "toggle_log");
     m_paShowLog->setCheckedState(i18n("Hide &Log Window"));
-    m_paDropTarget   = new KToggleAction(i18n("Drop &Target"),"tool_drop_target", 0, this, SLOT(slotToggleDropTarget()), coll, "drop_target");
+    m_paDropTarget   = new KAction(i18n("Show Drop &Target"),"tool_drop_target", 0, this, SLOT(slotToggleDropTarget()), coll, "drop_target");
+    m_paKonquerorIntegration = new KAction(i18n("Enable &KGet as Konqueror Download Manager"), "konqueror", 0, this, SLOT(slotKonquerorIntegration()), coll, "konqueror_integration");
+    if (ksettings.b_KonquerorIntegration) {
+        m_paKonquerorIntegration->setText(i18n("Disable &KGet as Konqueror Download Manager"));
+    }
 
     menuHelp = new KHelpMenu(this, KGlobal::instance()->aboutData());
     KStdAction::whatsThis(menuHelp, SLOT(contextHelpActivated()), coll, "whats_this");
@@ -393,7 +407,7 @@ actionCollection());
     statusBar()->insertFixedItem(i18n(" Transfers: %1 ").arg(99), ID_TOTAL_TRANSFERS);
     statusBar()->insertFixedItem(i18n(" Files: %1 ").arg(555), ID_TOTAL_FILES);
     statusBar()->insertFixedItem(i18n(" Size: %1 KB ").arg("134.56"), ID_TOTAL_SIZE);
-    statusBar()->insertFixedItem(i18n(" Time: 00:00:00 "), ID_TOTAL_TIME);
+    statusBar()->insertFixedItem(i18n(" Time: %1 ").arg(KIO::convertSeconds(0)), ID_TOTAL_TIME);
     statusBar()->insertFixedItem(i18n(" %1 KB/s ").arg("123.34"), ID_TOTAL_SPEED);
 
     setAutoSaveSettings( "MainWindow", false /*Settings takes care of size & pos & state */ );
@@ -688,7 +702,7 @@ void KMainWidget::slotQuit()
     for (; it.current(); ++it) {
         item = it.current();
         if (item->getStatus() == Transfer::ST_RUNNING && !ksettings.b_expertMode) {
-            if (KMessageBox::warningYesNo(this, i18n("Some transfers are still running.\nAre you sure you want to close KGet?"), i18n("Warning")) != KMessageBox::Yes) {
+            if (KMessageBox::warningContinueCancel(this, i18n("Some transfers are still running.\nAre you sure you want to quit KGet?"), i18n("Warning"), KStdGuiItem::quit()) != KMessageBox::Continue) {
 #ifdef _DEBUG
                 sDebugOut << endl;
 #endif
@@ -799,19 +813,19 @@ void KMainWidget::slotDeleteCurrent()
     {
         if ( selectedItems.count() > 1 )
         {
-            if (KMessageBox::questionYesNoList(this, i18n("Are you sure you want to delete these transfers?"),
+            if (KMessageBox::warningContinueCancelList(this, i18n("Are you sure you want to delete these transfers?"),
                                                itemNames, i18n("Question"),
-                                               KStdGuiItem::yes(),KStdGuiItem::no(),
+                                               KStdGuiItem::del(),
                                                QString("multiple_delete_transfer"))
-                != KMessageBox::Yes)
+                != KMessageBox::Continue)
                 return; // keep 'em
         }
         else
         {
-            if (KMessageBox::questionYesNo(this, i18n("Are you sure you want to delete this transfer?"),
-                                           i18n("Question"), KStdGuiItem::yes(),KStdGuiItem::no(),
+            if (KMessageBox::warningContinueCancel(this, i18n("Are you sure you want to delete this transfer?"),
+                                           i18n("Question"), KStdGuiItem::del(),
                                            QString("delete_transfer"))
-                != KMessageBox::Yes)
+                != KMessageBox::Continue)
                 return;
         }
     }
@@ -1093,7 +1107,7 @@ void KMainWidget::addTransferEx(const KURL& url, const KURL& destFile)
             //check if destination already exists
             if(KIO::NetAccess::exists(destURL, false, this))
             {
-                if (KMessageBox::warningYesNo(this,i18n("Destination file \n%1\nalready exists.\nDo you want to overwrite it?").arg( destURL.prettyURL()) )
+                if (KMessageBox::warningYesNo(this,i18n("Destination file \n%1\nalready exists.\nDo you want to overwrite it?").arg( destURL.prettyURL()), QString::null, i18n("Overwrite"), i18n("Do Not Overwrite") )
                                               == KMessageBox::Yes)
                 {
                     bDestisMalformed=false;
@@ -1120,6 +1134,7 @@ void KMainWidget::addTransferEx(const KURL& url, const KURL& destFile)
 
     // create a new transfer item
     Transfer *item = myTransferList->addTransfer(url, destURL);
+    KNotifyClient::event(kdock->winId(), "added", i18n("<i>%1</i> has been added.").arg(url.prettyURL()));
     item->updateAll(); // update the remaining fields
 
     if (ksettings.b_showIndividual)
@@ -1127,9 +1142,6 @@ void KMainWidget::addTransferEx(const KURL& url, const KURL& destFile)
 
     myTransferList->clearSelection();
 
-    if (ksettings.b_useSound) {
-        KAudioPlayer::play(ksettings.audioAdded);
-    }
     checkQueue();
 #ifdef _DEBUG
     sDebugOut << endl;
@@ -1172,7 +1184,7 @@ void KMainWidget::addTransfers( const KURL::List& src, const QString& destDir )
                 destURL.setFileName( fileName );
                 if(KIO::NetAccess::exists(destURL, false, this))
                 {
-                    if (KMessageBox::warningYesNo(this,i18n("Destination file \n%1\nalready exists.\nDo you want to overwrite it?").arg( destURL.prettyURL()) )
+                    if (KMessageBox::warningYesNo(this,i18n("Destination file \n%1\nalready exists.\nDo you want to overwrite it?").arg( destURL.prettyURL()), QString::null, i18n("Overwrite"), i18n("Do Not Overwrite") )
                         == KMessageBox::Yes)
                     {
                         SafeDelete::deleteFile( destURL );
@@ -1214,6 +1226,7 @@ void KMainWidget::addTransfers( const KURL::List& src, const QString& destDir )
     // dest is now finally the real destination directory for all the files
     dest.adjustPath(+1);
 
+    int numdl = 0;
     // create new transfer items
     KURL::List::ConstIterator it = urlsToDownload.begin();
     for ( ; it != urlsToDownload.end(); ++it )
@@ -1232,7 +1245,7 @@ void KMainWidget::addTransfers( const KURL::List& src, const QString& destDir )
 
         if(KIO::NetAccess::exists(destURL, false, this))
         {
-            if (KMessageBox::warningYesNo(this,i18n("Destination file \n%1\nalready exists.\nDo you want to overwrite it?").arg( destURL.prettyURL() ) )
+            if (KMessageBox::warningYesNo(this,i18n("Destination file \n%1\nalready exists.\nDo you want to overwrite it?").arg( destURL.prettyURL() ), QString::null, i18n("Overwrite"), i18n("Do Not Overwrite") )
                                           == KMessageBox::Yes)
             {
                 SafeDelete::deleteFile( destURL );
@@ -1241,13 +1254,14 @@ void KMainWidget::addTransfers( const KURL::List& src, const QString& destDir )
 
         Transfer *item = myTransferList->addTransfer(*it, destURL);
         item->updateAll(); // update the remaining fields
+
+        numdl++;
     }
 
+    KNotifyClient::event(kdock->winId(), "added", i18n("1 download has been added.", "%n downloads have been added.", numdl));
+
     myTransferList->clearSelection();
 
-    if (ksettings.b_useSound) {
-        KAudioPlayer::play(ksettings.audioAdded);
-    }
     checkQueue();
 }
 
@@ -1412,6 +1426,8 @@ void KMainWidget::slotStatusChanged(Transfer * item, int _operation)
     switch (_operation) {
 
     case Transfer::OP_FINISHED:
+    {
+        QString srcurl = item->getSrc().prettyURL();
         if (ksettings.b_removeOnSuccess && !item->keepDialogOpen() )
         {
             delete item;
@@ -1429,14 +1445,18 @@ void KMainWidget::slotStatusChanged(Transfer * item, int _operation)
                 slotQuit();
                 return;
             }
-            // play(ksettings.audioFinishedAll);
+            KNotifyClient::event(kdock->winId(), "finishedall", i18n("All the downloads are finished."));
+        }
+        else
+        {
+            KNotifyClient::event(kdock->winId(), "finished", i18n("<i>%1</i> successfully downloaded.").arg(srcurl));
         }
 
         if ( item )
             item->slotUpdateActions();
 
         break;
-
+    }
     case Transfer::OP_RESUMED:
         slotUpdateActions();
         item->slotUpdateActions();
@@ -1624,6 +1644,10 @@ void KMainWidget::slotPreferences()
 #endif
 }
 
+void KMainWidget::slotConfigureNotifications()
+{
+    KNotifyDialog::configure(this);
+}
 
 void KMainWidget::slotToggleLogWindow()
 {
@@ -1665,21 +1689,6 @@ void KMainWidget::slotToggleAnimation()
 #endif
 }
 
-
-void KMainWidget::slotToggleSound()
-{
-#ifdef _DEBUG
-    sDebugIn << endl;
-#endif
-
-    ksettings.b_useSound = !ksettings.b_useSound;
-
-#ifdef _DEBUG
-    sDebugOut << endl;
-#endif
-}
-
-
 void KMainWidget::slotToggleOfflineMode()
 {
 #ifdef _DEBUG
@@ -1823,13 +1832,16 @@ void KMainWidget::slotToggleDropTarget()
 #ifdef _DEBUG
     sDebugIn << endl;
 #endif
+    m_showDropTarget = !m_showDropTarget;
 
-    if (m_paDropTarget->isChecked()) {
+    if (m_showDropTarget) {
         kdrop->show();
         kdrop->updateStickyState();
+        m_paDropTarget->setText(i18n("Hide Drop &Target"));
     }
     else {
         kdrop->hide();
+        m_paDropTarget->setText(i18n("Show Drop &Target"));
     }
 
 
@@ -1839,6 +1851,33 @@ void KMainWidget::slotToggleDropTarget()
 }
 
 
+void KMainWidget::slotKonquerorIntegration()
+{
+#ifdef _DEBUG
+    sDebugIn << endl;
+#endif
+
+        bool bIsKonquiEnable=!ksettings.b_KonquerorIntegration;
+        ksettings.b_KonquerorIntegration=!ksettings.b_KonquerorIntegration;
+        KConfig cfg("konquerorrc", false, false);
+        cfg.setGroup("HTML Settings");
+        cfg.writePathEntry("DownloadManager",QString((bIsKonquiEnable)?"kget":""));
+        cfg.sync();
+        if (bIsKonquiEnable) 
+        {
+            m_paKonquerorIntegration->setText(i18n("Disable &KGet as Konqueror Download Manager"));
+        }
+        else
+        {
+            m_paKonquerorIntegration->setText(i18n("Enable &KGet as Konqueror Download Manager"));
+        }
+
+#ifdef _DEBUG
+    sDebugOut << endl;
+#endif
+}
+
+
 void KMainWidget::slotPopupMenu(Transfer * item)
 {
 #ifdef _DEBUG
@@ -2048,7 +2087,7 @@ void KMainWidget::updateStatusBar()
     int totalFiles = 0;
     KIO::filesize_t totalSize = 0;
     int totalSpeed = 0;
-    QTime remTime;
+    unsigned int remTime = 0;
 
     TransferIterator it(myTransferList);
 
@@ -2068,15 +2107,31 @@ void KMainWidget::updateStatusBar()
     statusBar()->changeItem(i18n(" Transfers: %1 ").arg(myTransferList->childCount()), ID_TOTAL_TRANSFERS);
     statusBar()->changeItem(i18n(" Files: %1 ").arg(totalFiles), ID_TOTAL_FILES);
     statusBar()->changeItem(i18n(" Size: %1 ").arg(KIO::convertSize(totalSize)), ID_TOTAL_SIZE);
-    statusBar()->changeItem(i18n(" Time: %1 ").arg(remTime.toString()), ID_TOTAL_TIME);
+    statusBar()->changeItem(i18n(" Time: %1 ").arg(KIO::convertSeconds(remTime)), ID_TOTAL_TIME);
     statusBar()->changeItem(i18n(" %1/s ").arg(KIO::convertSize(totalSpeed)), ID_TOTAL_SPEED);
+    //update size for each statusbar field
+    statusBar()->setItemFixed(ID_TOTAL_TRANSFERS, -1);
+    statusBar()->setItemFixed(ID_TOTAL_FILES, -1);
+    statusBar()->setItemFixed(ID_TOTAL_SIZE, -1);
+    statusBar()->setItemFixed(ID_TOTAL_TIME, -1);
+    statusBar()->setItemFixed(ID_TOTAL_SPEED, -1);
 
     if (kdock) {
-        tmpstr = i18n(" Transfers: %1 ").arg(myTransferList->childCount()) +
-                 i18n(" Files: %1 ").arg(totalFiles)+
-                 i18n(" Size: %1 ").arg(KIO::convertSize(totalSize))+
-                 i18n(" Time: %1 %2/s").arg(remTime.toString()).arg(KIO::convertSize(totalSpeed));
+        tmpstr = i18n("<b>Transfers:</b> %1 ").arg(myTransferList->childCount()) +
+                 i18n("<br /><b>Files:</b> %1 ").arg(totalFiles) +
+                 i18n("<br /><b>Size:</b> %1 ").arg(KIO::convertSize(totalSize)) +
+                 i18n("<br /><b>Time:</b> %1 ").arg(KIO::convertSeconds(remTime)) +
+                 i18n("<br /><b>Speed:</b> %1/s").arg(KIO::convertSize(totalSpeed));
         kdock->updateToolTip( tmpstr );
+        //trayicon changes if download is in progress
+        if (totalSpeed == 0)
+        {
+            kdock->changeIcon( "kget_dock" );
+        }
+        else
+        {
+            kdock->changeIcon( "kget_dock_download" );
+        }
     }
 #ifdef _DEBUG
     //sDebugOut << endl;
@@ -2097,7 +2152,7 @@ void KMainWidget::onlineDisconnect()
     if (!ksettings.b_expertMode) {
         if (KMessageBox::questionYesNo(this, i18n("Do you really want to disconnect?"),
                                        i18n("Question"),
-                                       KStdGuiItem::yes(), KStdGuiItem::no(),
+                                       i18n("Disconnect"), i18n("Stay Connected"),
                                        "kget_AutoOnlineDisconnect")
             != KMessageBox::Yes) {
             return;
@@ -2366,7 +2421,7 @@ bool KMainWidget::sanityChecksSuccessful( const KURL& url )
         else // transfer is finished, ask if we want to download again
         {
             if ( ksettings.b_expertMode ||
-                 (KMessageBox::questionYesNo(this, i18n("Already saved URL\n%1\nDownload again?").arg(url.prettyURL()), i18n("Question"))
+                 (KMessageBox::questionYesNo(this, i18n("Already saved URL\n%1\nDownload again?").arg(url.prettyURL()),i18n("Question"),i18n("Download Again"),KStdGuiItem::cancel() )
                      == KMessageBox::Yes) )
             {
                 transfer->slotRequestRemove();
@@ -2393,15 +2448,14 @@ bool KMainWidget::sanityChecksSuccessful( const KURL& url )
 
 bool KMainWidget::isDropTargetVisible() const
 {
-    return m_paDropTarget->isChecked();
+    return m_showDropTarget;
 }
 
 void KMainWidget::setDropTargetVisible( bool setVisible )
 {
     if ( setVisible != isDropTargetVisible() )
     {
-        m_paDropTarget->setChecked( !m_paDropTarget->isChecked() );
-        slotToggleDropTarget();
+        m_paDropTarget->activate();
     }
 }
 
@@ -2416,4 +2470,17 @@ bool KMainWidget::isOfflineMode() const
     return ksettings.b_offlineMode;
 }
 
+void KMainWidget::activateDropTarget()
+{
+    setDropTargetVisible( true );
+}
+
+void KMainWidget::slotShowMenubar()
+{
+    if(m_menubarAction->isChecked())
+        menuBar()->show();
+    else
+        menuBar()->hide();
+}
+
 #include "kmainwidget.moc"
