@@ -1,7 +1,7 @@
 /*
     chatview.cpp - Chat View
 
-    Copyright (c) 2002-2004 by Olivier Goffart       <ogoffart@tiscalinet.be>
+    Copyright (c) 2002-2004 by Olivier Goffart       <ogoffart @ kde.org>
     Copyright (c) 2002-2003 by Martijn Klingens      <klingens@kde.org>
 
     Kopete    (c) 2002-2004 by the Kopete developers <kopete-devel@kde.org>
@@ -22,7 +22,7 @@
 #include "chatmessagepart.h"
 #include "chattexteditpart.h"
 #include "kopetechatwindow.h"
-#include "kopetemessagemanager.h"
+#include "kopetechatsession.h"
 #include "kopetemetacontact.h"
 #include "kopetepluginmanager.h"
 #include "kopeteprefs.h"
@@ -78,11 +78,12 @@ ChatView::ChatView( Kopete::ChatSession *mgr, ChatWindowPlugin *parent, const ch
 	d->visibleMembers = false;
 	d->sendInProgress = false;
 
+
 	m_mainWindow = 0L;
 	membersDock = 0L;
-	m_tabBar = 0L;
 	membersStatus = Smart;
-	m_tabState=Normal;
+	m_tabState = Normal;
+
 
 	//FIXME: don't widgets start off hidden anyway?
 	hide();
@@ -103,7 +104,6 @@ ChatView::ChatView( Kopete::ChatSession *mgr, ChatWindowPlugin *parent, const ch
 
 	// FIXME: is this used these days? it seems totally unnecessary
 	connect( editPart(), SIGNAL( toggleToolbar(bool)), this, SLOT(slotToggleRtfToolbar(bool)) );
-	connect( editPart()->edit(), SIGNAL( textChanged() ) , this, SLOT( editPartTextChanged() ) );
 
 	connect( editPart(), SIGNAL( messageSent( Kopete::Message & ) ),
 	         this, SIGNAL( messageSent( Kopete::Message & ) ) );
@@ -281,13 +281,16 @@ void ChatView::raise( bool activate )
 		makeVisible();
 
 	if ( !KWin::windowInfo( m_mainWindow->winId(), NET::WMDesktop ).onAllDesktops() )
-		KWin::setOnDesktop( m_mainWindow->winId(), KWin::currentDesktop() );
+		if( KopetePrefs::prefs()->trayflashNotifySetCurrentDesktopToChatView() && activate )
+			KWin::setCurrentDesktop( KWin::windowInfo( m_mainWindow->winId(), NET::WMDesktop ).desktop() );
+		else
+			KWin::setOnDesktop( m_mainWindow->winId(), KWin::currentDesktop() );
 
 	if(m_mainWindow->isMinimized())
 	{
 		m_mainWindow->showNormal();
 	}
-	
+
 
 	m_mainWindow->raise();
 
@@ -323,14 +326,14 @@ void ChatView::makeVisible()
 		m_messagePart->keepScrolledDown();
 	}
 
-	
+
 
 	m_mainWindow->setActiveView( this );
 }
 
 bool ChatView::isVisible()
 {
-	return ( m_mainWindow && m_mainWindow->isVisible() && ( d->isActive || docked() ) );
+	return ( m_mainWindow && m_mainWindow->isVisible() );
 }
 
 bool ChatView::visibleMembersList()
@@ -388,7 +391,7 @@ bool ChatView::closeView( bool force )
 	return false;
 }
 
-void ChatView::setTabState( KopeteTabState newState )
+void ChatView::updateChatState( KopeteTabState newState )
 {
 	if ( newState == Undefined )
 		newState = m_tabState;
@@ -401,32 +404,7 @@ void ChatView::setTabState( KopeteTabState newState )
 
 	newState = m_remoteTypingMap.isEmpty() ? m_tabState : Typing ;
 
-	if( m_tabBar )
-	{
-		switch( newState )
-		{
-			case Highlighted:
-				m_tabBar->setTabColor( this, Qt::blue );
-				break;
-
-			case Message:
-				m_tabBar->setTabColor( this, Qt::red );
-				break;
-
-			case Changed:
-				m_tabBar->setTabColor( this, Qt::darkRed );
-				break;
-
-			case Typing:
-				m_tabBar->setTabColor( this, Qt::darkGreen );
-				break;
-
-			case Normal:
-			default:
-				m_tabBar->setTabColor( this, KGlobalSettings::textColor() );
-				break;
-		}
-	}
+	emit updateChatState( this, newState );
 
 	if( newState != Typing )
 	{
@@ -494,8 +472,8 @@ void ChatView::toggleMembersVisibility()
 
 void ChatView::placeMembersList( KDockWidget::DockPosition dp )
 {
-	kdDebug(14000) << k_funcinfo << "Members list policy " << membersStatus <<
-			", visible " << d->visibleMembers << endl;
+// 	kdDebug(14000) << k_funcinfo << "Members list policy " << membersStatus <<
+// 			", visible " << d->visibleMembers << endl;
 
 	if ( d->visibleMembers )
 	{
@@ -553,7 +531,6 @@ void ChatView::remoteTyping( const Kopete::Contact *contact, bool isTyping )
 
 	// Loop through the map, constructing a string of people typing
 	QStringList typingList;
-	QString statusTyping;
 	QPtrDictIterator<QTimer> it( m_remoteTypingMap );
 
 	for( ; it.current(); ++it )
@@ -563,17 +540,21 @@ void ChatView::remoteTyping( const Kopete::Contact *contact, bool isTyping )
 		typingList.append( c->metaContact() ? c->metaContact()->displayName() : ( nick.isEmpty() ? c->contactId() : nick ) );
 	}
 
-	statusTyping = typingList.join( QString::fromLatin1( ", " ) );
-
 	// Update the status area
 	if( !typingList.isEmpty() )
 	{
-		setStatusText( i18n( "%1 is typing a message", "%1 are typing a message", typingList.count() ).arg( statusTyping ) );
-		setTabState( Typing );
+		if ( typingList.count() == 1 )
+			setStatusText( i18n( "%1 is typing a message" ).arg( typingList.first() ) );
+		else
+		{
+			QString statusTyping = typingList.join( QString::fromLatin1( ", " ) );
+			setStatusText( i18n( "%1 is a list of names", "%1 are typing a message" ).arg( statusTyping ) );
+		}
+		updateChatState( Typing );
 	}
 	else
 	{
-		setTabState();
+		updateChatState();
 	}
 }
 
@@ -632,13 +613,13 @@ void ChatView::slotContactAdded(const Kopete::Contact *contact, bool suppress)
 		}
 	}
 
-	setTabState();
+	updateChatState();
 	emit updateStatusIcon( this );
 }
 
 void ChatView::slotContactRemoved( const Kopete::Contact *contact, const QString &reason, Kopete::Message::MessageFormat format, bool suppressNotification )
 {
-	kdDebug(14000) << k_funcinfo << endl;
+// 	kdDebug(14000) << k_funcinfo << endl;
 	if ( contact != m_manager->myself() )
 	{
 		m_remoteTypingMap.remove( const_cast<Kopete::Contact *>( contact ) );
@@ -661,7 +642,7 @@ void ChatView::slotContactRemoved( const Kopete::Contact *contact, const QString
 		}
 	}
 
-	setTabState();
+	updateChatState();
 	emit updateStatusIcon( this );
 }
 
@@ -672,7 +653,7 @@ QString& ChatView::caption() const
 
 void ChatView::setCaption( const QString &text, bool modified )
 {
-	kdDebug(14000) << k_funcinfo << endl;
+// 	kdDebug(14000) << k_funcinfo << endl;
 	QString newCaption = text;
 
 	//Save this caption
@@ -684,17 +665,13 @@ void ChatView::setCaption( const QString &text, bool modified )
 	//Call the original set caption
 	KDockMainWindow::setCaption( newCaption, false );
 
-	if( m_tabBar )
-	{
-		m_tabBar->setTabToolTip( this, QString::fromLatin1("<qt>%1</qt>").arg( d->captionText ) );
-		m_tabBar->setTabLabel( this, newCaption  );
-
-		//Blink icon if modified and not active
-		if( !d->isActive && modified )
-			setTabState( Changed );
-		else
-			setTabState();
-	}
+	emit updateChatTooltip( this, QString::fromLatin1("<qt>%1</qt>").arg( d->captionText ) );
+	emit updateChatLabel( this, newCaption );
+	//Blink icon if modified and not active
+	if( !d->isActive && modified )
+		updateChatState( Changed );
+	else
+		updateChatState();
 
 	//Tell the parent we changed our caption
 	emit( captionChanged( d->isActive ) );
@@ -704,22 +681,22 @@ void ChatView::appendMessage(Kopete::Message &message)
 {
 	remoteTyping( message.from(), false );
 
-	messagePart()->appendMessage( message );
+	messagePart()->appendMessage(message);
 	if( !d->isActive )
 	{
 		switch ( message.importance() )
 		{
 			case Kopete::Message::Highlight:
-				setTabState( Highlighted );
+				updateChatState( Highlighted );
 				break;
 			case Kopete::Message::Normal:
 				if ( message.direction() == Kopete::Message::Inbound )
 				{
-					setTabState( Message );
+					updateChatState( Message );
 					break;
 				}
 			default:
-				setTabState( Changed );
+				updateChatState( Changed );
 		}
 	}
 
@@ -740,15 +717,12 @@ void ChatView::slotMarkMessageRead()
 
 void ChatView::slotToggleRtfToolbar( bool enabled )
 {
-	if ( enabled )
-		m_mainWindow->toolBar( "formatToolBar" )->show();
-	else
-		m_mainWindow->toolBar( "formatToolBar" )->hide();
+	emit rtfEnabled( this, enabled );
 }
 
 void ChatView::slotContactStatusChanged( Kopete::Contact *contact, const Kopete::OnlineStatus &newStatus, const Kopete::OnlineStatus &oldStatus )
 {
-	kdDebug(14000) << k_funcinfo << endl;
+ 	kdDebug(14000) << k_funcinfo << contact << endl;
 	bool inhibitNotification = ( newStatus.status() == Kopete::OnlineStatus::Unknown ||
 	                             oldStatus.status() == Kopete::OnlineStatus::Unknown );
 	if ( contact && KopetePrefs::prefs()->showEvents() && !inhibitNotification )
@@ -776,19 +750,6 @@ void ChatView::slotContactStatusChanged( Kopete::Contact *contact, const Kopete:
 		}
 	}
 
-	if ( m_tabBar )
-	{
-		QPtrList<Kopete::Contact> chatMembers = m_manager->members();
-		Kopete::Contact *tempContact = 0L;
-		for ( QPtrListIterator<Kopete::Contact> it ( chatMembers ); it.current(); ++it )
-		{
-			if ( !tempContact || tempContact->onlineStatus() < (*it)->onlineStatus() )
-				tempContact = (*it);
-		}
-		if ( tempContact )
-			m_tabBar->setTabIconSet( this, m_manager->contactOnlineStatus( tempContact ).iconFor( tempContact ) );
-	}
-
 	// update the windows caption
 	slotChatDisplayNameChanged();
 	emit updateStatusIcon( this );
@@ -862,16 +823,11 @@ void ChatView::setActive( bool value )
 	d->isActive = value;
 	if ( d->isActive )
 	{
-		setTabState( Normal );
+		updateChatState( Normal );
 		emit( activated( static_cast<KopeteView*>(this) ) );
 	}
 }
 
-void ChatView::setTabBar( KTabWidget *tabBar )
-{
-	m_tabBar = tabBar;
-}
-
 void ChatView::slotRemoteTypingTimeout()
 {
 	// Remove the topmost timer from the list. Why does QPtrDict use void* keys and not typed keys? *sigh*
@@ -879,22 +835,6 @@ void ChatView::slotRemoteTypingTimeout()
 		remoteTyping( reinterpret_cast<const Kopete::Contact *>( QPtrDictIterator<QTimer>(m_remoteTypingMap).currentKey() ), false );
 }
 
-void ChatView::editPartTextChanged()
-{
-	QSyntaxHighlighter* qsh = m_editPart->edit()->syntaxHighlighter();
-	if ( !qsh )
-		return;
-	
-	KDictSpellingHighlighter* kdsh = dynamic_cast<KDictSpellingHighlighter*>( qsh );
-	if ( !kdsh )
-		return;
-
-	if ( kdsh->automatic() && kdsh->isActive() )
-		setStatusText( i18n("As-you-type spell checking enabled.") );
-	else if ( kdsh->automatic() && !kdsh->isActive() )
-		setStatusText( i18n("As-you-type spell checking disabled.") );
-}
-
 void ChatView::dragEnterEvent ( QDragEnterEvent * event )
 {
 	if( event->provides( "kopete/x-contact" ) )
