@@ -326,82 +326,140 @@ GaduSession::pubDirSearchClose()
 	searchSeqNr_=0;
 }
 
+unsigned int
+GaduSession::getPersonalInformation()
+{
+	gg_pubdir50_t searchRequest;
+	unsigned int seqNr;
+
+	if ( isConnected() == false ) {
+		return 0;
+	}
+
+	searchRequest = gg_pubdir50_new( GG_PUBDIR50_READ );
+	if ( !searchRequest ) {
+		return 0;
+	}
+
+	seqNr = gg_pubdir50( session_, searchRequest );
+	gg_pubdir50_free( searchRequest );
+
+	return seqNr;
+}
+
 bool
-GaduSession::pubDirSearch(QString& name, QString& surname, QString& nick, int UIN, QString& city,
-                            int gender, int ageFrom, int ageTo, bool onlyAlive)
+GaduSession::publishPersonalInformation( ResLine& d )
+{
+	gg_pubdir50_t r;
+	
+	if ( !session_ ) {
+		return 0;
+        }		
+	
+	r = gg_pubdir50_new( GG_PUBDIR50_WRITE );
+
+	if ( d.firstname.length() )
+		gg_pubdir50_add( r, GG_PUBDIR50_FIRSTNAME, 
+			 (const char *)((const char*)textcodec->fromUnicode( d.firstname ) ) );
+        if ( d.surname.length() )
+		gg_pubdir50_add( r, GG_PUBDIR50_LASTNAME,
+			(const char *)((const char*)textcodec->fromUnicode( d.surname ) ) );
+	if ( d.nickname.length() )
+		gg_pubdir50_add( r, GG_PUBDIR50_NICKNAME,
+			(const char *)((const char*)textcodec->fromUnicode( d.nickname ) ) );
+	if ( d.age.length() )
+		gg_pubdir50_add( r, GG_PUBDIR50_BIRTHYEAR,
+			(const char *)((const char*)textcodec->fromUnicode( d.age ) ) );
+        if ( d.city.length() )
+        	gg_pubdir50_add( r, GG_PUBDIR50_CITY,
+			(const char *)((const char*)textcodec->fromUnicode( d.city ) ) );
+	if ( d.meiden.length() )
+		gg_pubdir50_add( r, GG_PUBDIR50_FAMILYNAME,
+			(const char *)((const char*)textcodec->fromUnicode( d.meiden ) ) );
+        if ( d.orgin.length() )
+		gg_pubdir50_add( r, GG_PUBDIR50_FAMILYCITY,
+			(const char *)((const char*)textcodec->fromUnicode( d.orgin ) ) );
+	if ( d.gender.length() == 1 )
+		gg_pubdir50_add( r, GG_PUBDIR50_GENDER, 
+			(const char *)((const char*)textcodec->fromUnicode( d.gender ) ) );
+        
+	gg_pubdir50( session_, r );
+        
+	gg_pubdir50_free( r );
+
+	return true;
+}
+
+unsigned int
+GaduSession::pubDirSearch(  ResLine& query, int ageFrom, int ageTo, bool onlyAlive )
 {
 	QString bufYear;
-	gg_pubdir50_t searchRequest_;
+	unsigned int reqNr;
+	gg_pubdir50_t searchRequest;
 
 	if ( !session_ ) {
-		return false;
+		return 0;
 	}
 
-	searchRequest_ = gg_pubdir50_new( GG_PUBDIR50_SEARCH_REQUEST );
-	if ( !searchRequest_ ) {
-		return false;
+	searchRequest = gg_pubdir50_new( GG_PUBDIR50_SEARCH_REQUEST );
+	if ( !searchRequest ) {
+		return 0;
 	}
 
-	if ( !UIN ) {
-		if (name.length()) {
-			gg_pubdir50_add( searchRequest_, GG_PUBDIR50_FIRSTNAME,
-						(const char*)textcodec->fromUnicode( name ) );
+	if ( query.uin == 0 ) {
+		if (query.firstname.length()) {
+			gg_pubdir50_add( searchRequest, GG_PUBDIR50_FIRSTNAME,
+						(const char*)textcodec->fromUnicode( query.firstname ) );
 		}
-		if ( surname.length() ) {
-			gg_pubdir50_add( searchRequest_, GG_PUBDIR50_LASTNAME,
-						(const char*)textcodec->fromUnicode( surname ) );
+		if ( query.surname.length() ) {
+			gg_pubdir50_add( searchRequest, GG_PUBDIR50_LASTNAME,
+						(const char*)textcodec->fromUnicode( query.surname ) );
 		}
-		if ( nick.length() ) {
-			gg_pubdir50_add( searchRequest_, GG_PUBDIR50_NICKNAME,
-						(const char*)textcodec->fromUnicode( nick ) );
+		if ( query.nickname.length() ) {
+			gg_pubdir50_add( searchRequest, GG_PUBDIR50_NICKNAME,
+						(const char*)textcodec->fromUnicode( query.nickname ) );
 		}
-		if ( city.length() ) {
-			gg_pubdir50_add( searchRequest_, GG_PUBDIR50_CITY,
-						(const char*)textcodec->fromUnicode( city ) );
+		if ( query.city.length() ) {
+			gg_pubdir50_add( searchRequest, GG_PUBDIR50_CITY,
+						(const char*)textcodec->fromUnicode( query.city ) );
 		}
 		if ( ageFrom || ageTo ) {
 			QString yearFrom = QString::number( QDate::currentDate().year() - ageFrom );
 			QString yearTo = QString::number( QDate::currentDate().year() - ageTo );
 
 			if ( ageFrom && ageTo ) {
-				gg_pubdir50_add( searchRequest_, GG_PUBDIR50_BIRTHYEAR,
+				gg_pubdir50_add( searchRequest, GG_PUBDIR50_BIRTHYEAR,
 							(const char*)textcodec->fromUnicode( yearFrom + " " + yearTo ) );
 			}
 			if ( ageFrom ) {
-				gg_pubdir50_add( searchRequest_, GG_PUBDIR50_BIRTHYEAR,
+				gg_pubdir50_add( searchRequest, GG_PUBDIR50_BIRTHYEAR,
 							(const char*)textcodec->fromUnicode( yearFrom ) );
 			}
 			else {
-				gg_pubdir50_add( searchRequest_, GG_PUBDIR50_BIRTHYEAR,
+				gg_pubdir50_add( searchRequest, GG_PUBDIR50_BIRTHYEAR,
 							(const char*)textcodec->fromUnicode( yearTo ) );
 			}
 		}
 
-		switch ( gender ) {
-			case 1:
-				gg_pubdir50_add( searchRequest_, GG_PUBDIR50_GENDER, GG_PUBDIR50_GENDER_MALE );
-			break;
-			case 2:
-				gg_pubdir50_add( searchRequest_, GG_PUBDIR50_GENDER, GG_PUBDIR50_GENDER_FEMALE );
-			break;
+		if ( query.gender.length() == 1 ) {
+                	gg_pubdir50_add( searchRequest, GG_PUBDIR50_GENDER,
+				(const char *)((const char*)textcodec->fromUnicode( query.gender ) ) );
 		}
 
 		if ( onlyAlive ) {
-			gg_pubdir50_add( searchRequest_, GG_PUBDIR50_ACTIVE, GG_PUBDIR50_ACTIVE_TRUE );
+			gg_pubdir50_add( searchRequest, GG_PUBDIR50_ACTIVE, GG_PUBDIR50_ACTIVE_TRUE );
 		}
 	}
 	// otherwise we are looking only for one fellow with this nice UIN
 	else{
-		gg_pubdir50_add( searchRequest_, GG_PUBDIR50_UIN, QString::number( UIN ).ascii() );
+		gg_pubdir50_add( searchRequest, GG_PUBDIR50_UIN, QString::number( query.uin ).ascii() );
 	}
 
-	gg_pubdir50_add( searchRequest_, GG_PUBDIR50_START, QString::number(searchSeqNr_).ascii() );
+	gg_pubdir50_add( searchRequest, GG_PUBDIR50_START, QString::number( searchSeqNr_ ).ascii() );
+	reqNr = gg_pubdir50( session_, searchRequest );
+	gg_pubdir50_free( searchRequest );
 
-	gg_pubdir50( session_, searchRequest_ );
-
-	gg_pubdir50_free( searchRequest_ );
-
-	return true;
+	return reqNr;
 }
 
 void
@@ -413,15 +471,22 @@ GaduSession::sendResult( gg_pubdir50_t result )
 
 	count = gg_pubdir50_count( result );
 
-	for ( i = 0; i < count; i++ ) {
+	if ( !count ) {
+		kdDebug(14100) << "there was nothing found in public directory for requested details" << endl;
+	}
 
-		resultLine.uin		= textcodec->toUnicode( gg_pubdir50_get( result, i, GG_PUBDIR50_UIN ) );
-		resultLine.firstname	= textcodec->toUnicode( gg_pubdir50_get( result, i, GG_PUBDIR50_FIRSTNAME ));
-		resultLine.surname	= textcodec->toUnicode( gg_pubdir50_get( result, i, GG_PUBDIR50_LASTNAME ));
-		resultLine.nickname	= textcodec->toUnicode( gg_pubdir50_get(result, i, GG_PUBDIR50_NICKNAME ) );
+	for ( i = 0; i < count; i++ ) {
+		resultLine.uin		= QString( gg_pubdir50_get( result, i, GG_PUBDIR50_UIN ) ).toInt();
+		resultLine.firstname	= textcodec->toUnicode( gg_pubdir50_get( result, i, GG_PUBDIR50_FIRSTNAME ) );
+		resultLine.surname	= textcodec->toUnicode( gg_pubdir50_get( result, i, GG_PUBDIR50_LASTNAME ) );
+		resultLine.nickname	= textcodec->toUnicode( gg_pubdir50_get( result, i, GG_PUBDIR50_NICKNAME ) );
 		resultLine.age		= textcodec->toUnicode( gg_pubdir50_get( result, i, GG_PUBDIR50_BIRTHYEAR ) );
 		resultLine.city		= textcodec->toUnicode( gg_pubdir50_get( result, i, GG_PUBDIR50_CITY ) );
 		QString stat		= textcodec->toUnicode( gg_pubdir50_get( result, i, GG_PUBDIR50_STATUS ) );
+		resultLine.orgin	= textcodec->toUnicode( gg_pubdir50_get( result, i, GG_PUBDIR50_FAMILYCITY ) );
+		resultLine.meiden	= textcodec->toUnicode( gg_pubdir50_get( result, i, GG_PUBDIR50_FAMILYNAME ) );
+		resultLine.gender	= textcodec->toUnicode( gg_pubdir50_get( result, i, GG_PUBDIR50_GENDER ) );
+
 		resultLine.status	= stat.toInt();
 		age = resultLine.age.toInt();
 		if ( age ) {
@@ -435,8 +500,7 @@ GaduSession::sendResult( gg_pubdir50_t result )
 	}
 
 	searchSeqNr_ = gg_pubdir50_next( result );
-
-	emit pubDirSearchResult( sres );
+	emit pubDirSearchResult( sres, gg_pubdir50_seq( result ) );
 }
 
 void
@@ -585,13 +649,17 @@ GaduSession::failureDescription( gg_failure_t f )
 void
 GaduSession::notify60( gg_event* event )
 {
-	KGaduNotifyList nl;
-	KGaduNotify* gn;
+	KGaduNotify* gn = NULL;
 	unsigned int n;
-	nl.setAutoDelete( TRUE );
+	
+	if ( event->event.notify60[0].uin ) {
+		gn = new KGaduNotify;
+	}
+	else {
+		return;
+	}
 
 	for( n=0 ; event->event.notify60[n].uin ; n++ ) {
-		gn = new KGaduNotify;
 		gn->contact_id	= event->event.notify60[n].uin;
 		gn->status	= event->event.notify60[n].status;
 		gn->remote_ip.setAddress( ntohl( event->event.notify60[n].remote_ip ) );
@@ -605,11 +673,9 @@ GaduSession::notify60( gg_event* event )
 		gn->version	= event->event.notify60[n].version;
 		gn->image_size	= event->event.notify60[n].image_size;
 		gn->description	= textcodec->toUnicode( event->event.notify60[n].descr );
-		nl.append( gn );
-	}
-	if ( n ) {
-		emit notify( &nl );
+		emit contactStatusChanged( gn );	
 	}
+	delete gn;
 }
 
 void
@@ -644,7 +710,8 @@ GaduSession::checkDescriptor()
 				// TODO: DCC CONNECTION
 				emit incomingCtcp( event->event.msg.sender );
 			}
-			if ( event->event.msg.msgclass & GG_CLASS_MSG || event->event.msg.msgclass & GG_CLASS_CHAT ) {
+
+			if ( (event->event.msg.msgclass & GG_CLASS_MSG) || (event->event.msg.msgclass & GG_CLASS_CHAT) ) {
 				gaduMessage.message =
 					textcodec->toUnicode((const char*)event->event.msg.message);
 				gaduMessage.sender_id = event->event.msg.sender;
