@@ -3,9 +3,9 @@
 
     Copyright (c) 2002      by Nick Betcher <nbetcher@kde.org>
     Copyright (c) 2003-2004 by Jason Keirstead <jason@keirstead.org>
-    Copyright (c) 2003-2004 by Michel Hermier <michel.hermier@wanadoo.fr>
+    Copyright (c) 2003-2005 by Michel Hermier <michel.hermier@wanadoo.fr>
 
-    Kopete    (c) 2002-2004 by the Kopete developers <kopete-devel@kde.org>
+    Kopete    (c) 2002-2005 by the Kopete developers <kopete-devel@kde.org>
 
     *************************************************************************
     *                                                                       *
@@ -21,72 +21,42 @@
 #include "irccontact.h"
 #include "irccontactmanager.h"
 #include "ircprotocol.h"
-#include "channellist.h"
 
 #include "ircservercontact.h"
 #include "ircchannelcontact.h"
 #include "ircusercontact.h"
 
+#include "channellistdialog.h"
+
+#include "kircengine.h"
+
+#include "kopeteaccountmanager.h"
 #include "kopeteaway.h"
 #include "kopeteawayaction.h"
-#include "kopeteuiglobal.h"
+#include "kopetecommandhandler.h"
 #include "kopetecontactlist.h"
 #include "kopetemetacontact.h"
-#include "kopeteaccountmanager.h"
-#include "kopetecommandhandler.h"
+#include "kopeteuiglobal.h"
 #include "kopeteview.h"
 #include "kopetepassword.h"
 
 #include <kaction.h>
 #include <kaboutdata.h>
 #include <kapplication.h>
+#include <kconfig.h>
+#include <kcompletionbox.h>
 #include <kdebug.h>
+#include <kglobal.h>
+#include <kinputdialog.h>
 #include <klineedit.h>
 #include <klineeditdlg.h>
-#include <kcompletionbox.h>
 #include <kmessagebox.h>
-#include <kpopupmenu.h>
-#include <kconfig.h>
-#include <kglobal.h>
 #include <knotifyclient.h>
+#include <kpopupmenu.h>
 
 #include <qlayout.h>
 #include <qtimer.h>
 
-ChannelListDialog::ChannelListDialog(KIRC::Engine *engine, const QString &caption, QObject *target, const char* slotJoinChan)
-	: KDialogBase(Kopete::UI::Global::mainWidget(), "channel_list_widget", false, caption, Close)
-{
-	m_engine = engine;
-	m_list = new ChannelList( this, engine );
-
-	connect( m_list, SIGNAL( channelDoubleClicked( const QString & ) ),
-		target, slotJoinChan );
-
-	connect( m_list, SIGNAL( channelDoubleClicked( const QString & ) ),
-		this, SLOT( slotChannelDoubleClicked( const QString & ) ) );
-
-	new QHBoxLayout( m_list, 0, spacingHint() );
-
-	setInitialSize( QSize( 500, 400 ) );
-	setMainWidget( m_list );
-	show();
-}
-
-void ChannelListDialog::clear()
-{
-	m_list->clear();
-}
-
-void ChannelListDialog::search()
-{
-	m_list->search();
-}
-
-void ChannelListDialog::slotChannelDoubleClicked( const QString & )
-{
-	close();
-}
-
 const QString IRCAccount::CONFIG_CODECMIB = QString::fromLatin1("Codec");
 const QString IRCAccount::CONFIG_NETWORKNAME = QString::fromLatin1("NetworkName");
 const QString IRCAccount::CONFIG_NICKNAME = QString::fromLatin1("NickName");
@@ -94,7 +64,7 @@ const QString IRCAccount::CONFIG_USERNAME = QString::fromLatin1("UserName");
 const QString IRCAccount::CONFIG_REALNAME = QString::fromLatin1("RealName");
 
 IRCAccount::IRCAccount(IRCProtocol *protocol, const QString &accountId, const QString &autoChan, const QString& netName, const QString &nickName)
-	: Kopete::PasswordedAccount(protocol, accountId, 0, true), autoConnect( autoChan )
+	: Kopete::PasswordedAccount(protocol, accountId, 0, true), autoConnect( autoChan ), commandSource(0)
 {
 	m_manager = 0L;
 	m_channelList = 0L;
@@ -236,6 +206,11 @@ IRCAccount::IRCAccount(IRCProtocol *protocol, const QString &accountId, const QS
 	setMyself( m_contactManager->mySelf() );
 	setAccountLabel( QString::fromLatin1("%1@%2").arg(mNickName,networkName) );
 	m_myServer = m_contactManager->myServer();
+
+	m_joinChannelAction = new KAction ( i18n("Join Channel..."), QString::null, 0, this,
+				SLOT(slotJoinChannel()), this);
+	m_searchChannelAction = new KAction ( i18n("Search Channels..."), QString::null, 0, this,
+				SLOT(slotSearchChannels()), this);
 }
 
 IRCAccount::~IRCAccount()
@@ -249,10 +224,15 @@ void IRCAccount::slotNickInUse( const QString &nick )
 	QString altNickName = altNick();
 	if( triedAltNick || altNickName.isEmpty() )
 	{
-		QString newNick = KLineEditDlg::getText(i18n( "IRC Plugin"),
-			i18n("The nickname %1 is already in use. Please enter an alternate nickname:").arg(nick), nick);
+		QString newNick = KInputDialog::getText(
+				i18n("IRC Plugin"),
+				i18n("The nickname %1 is already in use. Please enter an alternate nickname:").arg(nick),
+				nick);
 
-		m_engine->nick(newNick);
+		if (newNick.isNull())
+			disconnect();
+		else
+			m_engine->nick(newNick);
 	}
 	else
 	{
@@ -269,7 +249,6 @@ void IRCAccount::slotNickInUseAlert( const QString &nick )
 void IRCAccount::setAltNick( const QString &altNick )
 {
 	configGroup()->writeEntry(QString::fromLatin1( "altNick" ), altNick);
-	configGroup()->sync();
 }
 
 const QString IRCAccount::altNick() const
@@ -281,7 +260,6 @@ void IRCAccount::setAutoShowServerWindow( bool show )
 {
 	autoShowServerWindow = show;
 	configGroup()->writeEntry(QString::fromLatin1( "AutoShowServerWindow" ), autoShowServerWindow);
-	configGroup()->sync();
 }
 
 const QString IRCAccount::networkName() const
@@ -296,7 +274,6 @@ void IRCAccount::setUserName( const QString &userName )
 {
 	m_engine->setUserName(userName);
 	configGroup()->writeEntry(CONFIG_USERNAME, userName);
-	configGroup()->sync();
 }
 
 const QString IRCAccount::userName() const
@@ -308,7 +285,6 @@ void IRCAccount::setRealName( const QString &userName )
 {
 	m_engine->setRealName(userName);
 	configGroup()->writeEntry(CONFIG_REALNAME, userName);
-	configGroup()->sync();
 }
 
 const QString IRCAccount::realName() const
@@ -323,7 +299,6 @@ void IRCAccount::setNetwork( const QString &network )
 	{
 		m_network = net;
 		configGroup()->writeEntry(CONFIG_NETWORKNAME, network);
-		configGroup()->sync();
 		setAccountLabel(network);
 	}
 	else
@@ -340,7 +315,6 @@ void IRCAccount::setNickName( const QString &nick )
 {
 	mNickName = nick;
 	configGroup()->writeEntry(CONFIG_NICKNAME, mNickName);
-	configGroup()->sync();
 
 	if( mySelf() )
 		mySelf()->setNickName( mNickName );
@@ -351,7 +325,6 @@ void IRCAccount::setCodec( QTextCodec *codec )
 {
 	mCodec = codec;
 	configGroup()->writeEntry(CONFIG_CODECMIB, codec->mibEnum());
-	configGroup()->sync();
 
 	if( mCodec )
 		m_engine->setDefaultCodec( mCodec );
@@ -366,14 +339,12 @@ QTextCodec *IRCAccount::codec() const
 void IRCAccount::setDefaultPart( const QString &defaultPart )
 {
 	configGroup()->writeEntry( QString::fromLatin1( "defaultPart" ), defaultPart );
-	configGroup()->sync();
 }
 
 // FIXME: Move this to a dictionnary
 void IRCAccount::setDefaultQuit( const QString &defaultQuit )
 {
 	configGroup()->writeEntry( QString::fromLatin1( "defaultQuit" ), defaultQuit );
-	configGroup()->sync();
 }
 
 // FIXME: Move this to a dictionnary
@@ -402,9 +373,7 @@ void IRCAccount::setCustomCtcpReplies( const QMap< QString, QString > &replies )
 		val.append( QString::fromLatin1("%1=%2").arg( it.key() ).arg( it.data() ) );
 	}
 
-	KConfigGroup *config = configGroup();
-	config->writeEntry( "CustomCtcp", val );
-	config->sync();
+	configGroup()->writeEntry( "CustomCtcp", val );
 }
 
 const QMap< QString, QString > IRCAccount::customCtcpReplies() const
@@ -412,8 +381,7 @@ const QMap< QString, QString > IRCAccount::customCtcpReplies() const
 	QMap< QString, QString > replies;
 	QStringList replyList;
 
-	KConfigGroup *config = configGroup() ;
-	replyList = config->readListEntry( "CustomCtcp" );
+	replyList = configGroup()->readListEntry( "CustomCtcp" );
 
 	for( QStringList::Iterator it = replyList.begin(); it != replyList.end(); ++it )
 		replies[ (*it).section('=', 0, 0 ) ] = (*it).section('=', 1 );
@@ -423,15 +391,12 @@ const QMap< QString, QString > IRCAccount::customCtcpReplies() const
 
 void IRCAccount::setConnectCommands( const QStringList &commands ) const
 {
-	KConfigGroup *config = configGroup() ;
-	config->writeEntry( "ConnectCommands", commands );
-	config->sync();
+	configGroup()->writeEntry( "ConnectCommands", commands );
 }
 
 const QStringList IRCAccount::connectCommands() const
 {
-	KConfigGroup *config =  configGroup() ;
-	return config->readListEntry( "ConnectCommands" );
+	return configGroup()->readListEntry( "ConnectCommands" );
 }
 
 void IRCAccount::setMessageDestinations( int serverNotices, int serverMessages,
@@ -442,7 +407,6 @@ void IRCAccount::setMessageDestinations( int serverNotices, int serverMessages,
 	config->writeEntry( "ServerMessages", serverMessages );
 	config->writeEntry( "InformationReplies", informationReplies );
 	config->writeEntry( "ErrorMessages", errorMessages );
-	config->sync();
 
 	m_serverNotices = (MessageDestination)serverNotices;
 	m_serverMessages = (MessageDestination)serverMessages;
@@ -456,10 +420,13 @@ KActionMenu *IRCAccount::actionMenu()
 
 	KActionMenu *mActionMenu = Kopete::Account::actionMenu();
 
+	m_joinChannelAction->setEnabled( isConnected() );
+	m_searchChannelAction->setEnabled( isConnected() );
+
 	mActionMenu->popupMenu()->insertSeparator();
-	mActionMenu->insert( new KAction ( i18n("Join Channel..."), "", 0, this, SLOT(slotJoinChannel()), mActionMenu ) );
-	mActionMenu->insert( new KAction ( i18n("Search Channels..."), "", 0, this, SLOT(slotSearchChannels()), mActionMenu ) );
-	mActionMenu->insert( new KAction ( i18n("Show Server Window"), "", 0, this, SLOT(slotShowServerWindow()), mActionMenu ) );
+	mActionMenu->insert(m_joinChannelAction);
+	mActionMenu->insert(m_searchChannelAction);
+	mActionMenu->insert( new KAction ( i18n("Show Server Window"), QString::null, 0, this, SLOT(slotShowServerWindow()), mActionMenu ) );
 
 	if( m_engine->isConnected() && m_engine->useSSL() )
 	{
@@ -686,7 +653,8 @@ void IRCAccount::slotShowServerWindow()
 
 bool IRCAccount::isConnected()
 {
-	return ( myself()->onlineStatus().status() != Kopete::OnlineStatus::Offline );
+//	return ( myself()->onlineStatus().status() != Kopete::OnlineStatus::Offline );
+	return m_engine->isConnected();
 }
 
 void IRCAccount::setOnlineStatus( const Kopete::OnlineStatus& status , const QString &reason )
@@ -761,14 +729,23 @@ void IRCAccount::slotJoinNamedChannel(const QString &chan)
 	contactManager()->findChannel(chan)->startChat();
 }
 
+void IRCAccount::setCurrentCommandSource( Kopete::ChatSession *session )
+{
+	commandSource = session;
+}
+
+Kopete::ChatSession *IRCAccount::currentCommandSource()
+{
+	return commandSource;
+}
+
 void IRCAccount::slotJoinChannel()
 {
 	if (!isConnected())
 		return;
 
-	KConfig *config = kapp->config();
-	config->setGroup( QString::fromLatin1("Account_IRCProtocol_") + accountId());
-	QStringList chans = config->readListEntry("Recent Channel list");
+	QStringList chans = configGroup()->readListEntry( "Recent Channel list" );
+	//kdDebug(14120) << "Recent channel list from config: " << chans << endl;
 
 	KLineEditDlg dlg(
 		i18n("Please enter name of the channel you want to join:"),
@@ -776,34 +753,37 @@ void IRCAccount::slotJoinChannel()
 		Kopete::UI::Global::mainWidget()
 	);
 
-	if (!chans.isEmpty())
-	{
-		dlg.lineEdit()->setCompletedItems( chans );
-		dlg.lineEdit()->setCompletionMode( KGlobalSettings::CompletionPopupAuto );
-	}
+	KCompletion comp;
+	comp.insertItems( chans );
 
-	if (dlg.exec() == QDialog::Accepted)
+	dlg.lineEdit()->setCompletionObject( &comp );
+	dlg.lineEdit()->setCompletionMode( KGlobalSettings::CompletionPopup );
+
+	while( true )
 	{
+		if( dlg.exec() != QDialog::Accepted )
+			break;
+
 		QString chan = dlg.text();
-		chans = dlg.lineEdit()->completionBox()->items();
-		chans.append( chan );
+		if( chan.isNull() )
+			break;
 
-		if( !chan.isNull() )
+		if( KIRC::Entity::isChannel( chan ) )
 		{
-			if( KIRC::Entity::isChannel( chan ) )
-				contactManager()->findChannel( chan )->startChat();
-			else
-				KMessageBox::error( Kopete::UI::Global::mainWidget(),
-					i18n("\"%1\" is an invalid channel. Channels must start with '#', '!', '+', or '&'.")
-					.arg(chan), i18n("IRC Plugin")
-				);
+			contactManager()->findChannel( chan )->startChat();
+
+			// push the joined channel to first in list
+			chans.remove( chan );
+			chans.prepend( chan );
+
+			configGroup()->writeEntry( "Recent Channel list", chans );
+			break;
 		}
 
-		if( !chans.isEmpty() )
-		{
-			config->writeEntry( "Recent Channel list", chans );
-			config->sync();
-		}
+		KMessageBox::error( Kopete::UI::Global::mainWidget(),
+			i18n("\"%1\" is an invalid channel. Channels must start with '#', '!', '+', or '&'.").arg(chan),
+			i18n("IRC Plugin")
+			);
 	}
 }
 
@@ -887,6 +867,33 @@ IRCServerContact *IRCAccount::myServer() const
 	return m_myServer;
 }
 
+IRCContact *IRCAccount::getContact(const QString &name, Kopete::MetaContact *metac)
+{
+	return getContact(m_engine->getEntity(name), metac);
+}
+
+IRCContact *IRCAccount::getContact(KIRC::EntityPtr entity, Kopete::MetaContact *metac)
+{
+	IRCContact *contact = 0;
+
+	#pragma warning Do the search code here.
+
+	if (!contact)
+	{
+		#pragma warning Make a temporary meta contact if metac is null
+		contact = new IRCContact(this, entity, metac);
+		m_contacts.append(contact);
+	}
+
+	QObject::connect(contact, SIGNAL(destroyed(IRCContact *)), SLOT(destroyed(IRCContact *)));
+	return contact;
+}
+
+void IRCAccount::destroyed(IRCContact *contact)
+{
+	m_contacts.remove(contact);
+}
+
 #include "ircaccount.moc"
 
 // vim: set noet ts=4 sts=4 sw=4:
