@@ -53,7 +53,7 @@ Message::Message()
 }
 
 Message::Message(const Message &obj)
-	: m_ctcpMessage(0)
+        : QObject(), m_ctcpMessage(0)
 {
 	m_raw = obj.m_raw;
 
@@ -163,7 +163,17 @@ Message Message::parse(Engine *engine, const QTextCodec *codec, bool *parseSucce
 		if( length > -1 )
 		{
 			raw.resize( length );
-			raw.replace("\r\n",""); //remove the trailling \r\n if any(there must be in fact)
+
+			// Remove trailing '\r\n' or '\n'.
+			//
+			// Some servers send '\n' instead of '\r\n' that the RFCs say they should be sending.
+
+			if (length > 1 && raw.at(length-2) == '\n') {
+				raw.at(length-2) = '\0';
+			}
+			if (length > 2 && raw.at(length-3) == '\r') {
+				raw.at(length-3) = '\0';
+			}
 
 			kdDebug(14121) << "<< " << raw << endl;
 
@@ -203,10 +213,8 @@ QString Message::unquote(const QString &str)
 {
 	QString tmp = str;
 
-	char b[3];
-	b[0] = 20; b[1] = 20; b[2] = '\0';
-	char b2[2];
-	b2[0] = (char)20; b2[1] = '\0';
+	char b[3] = { 020, 020, '\0' };
+	const char b2[2] = { 020, '\0' };
 
 	tmp.replace( b, b2 );
 	b[1] = 'r';
@@ -240,7 +248,7 @@ bool Message::matchForIRCRegExp(const QCString &line, const QTextCodec *codec, M
 	if(matchForIRCRegExp(m_IRCCommandType1, codec, line, message))
 		return true;
 #ifdef _IRC_STRICTNESS_
-	if(!matchForIRCRegExp(m_IRCCommandType2, codec, line, message)
+	if(!matchForIRCRegExp(m_IRCCommandType2, codec, line, message))
 		return true;
 #endif // _IRC_STRICTNESS_
 	return false;
@@ -257,25 +265,32 @@ bool Message::matchForIRCRegExp(QRegExp &regexp, const QTextCodec *codec, const
 		msg.m_command = unquote(regexp.cap(2));
 		msg.m_args = QStringList::split(' ', regexp.cap(3));
 
-		QCString suffix = regexp.cap(4).latin1();
+		QCString suffix = codec->fromUnicode(unquote(regexp.cap(4)));
 		if (!suffix.isNull() && suffix.length() > 0)
 		{
-			if (extractCtcpCommand(suffix, msg.m_ctcpRaw, codec))
+			QCString ctcpRaw;
+			if (extractCtcpCommand(suffix, ctcpRaw))
 			{
-				msg.m_ctcpMessage = new Message();
-				msg.m_ctcpMessage->m_raw = msg.m_ctcpRaw.latin1();
+				msg.m_ctcpRaw = codec->toUnicode(ctcpRaw); 
 
-				int space = msg.m_ctcpRaw.find(' ');
+				msg.m_ctcpMessage = new Message();
+				msg.m_ctcpMessage->m_raw = codec->fromUnicode(ctcpUnquote(msg.m_ctcpRaw));
+
+				int space = ctcpRaw.find(' ');
 				if (!matchForIRCRegExp(msg.m_ctcpMessage->m_raw, codec, *msg.m_ctcpMessage))
 				{
+					QCString command;
 					if (space > 0)
-						msg.m_ctcpMessage->m_command = msg.m_ctcpRaw.mid(0, space).upper();
+						command = ctcpRaw.mid(0, space).upper();
 					else
-						msg.m_ctcpMessage->m_command = msg.m_ctcpRaw.upper();
+						command = ctcpRaw.upper();
+					msg.m_ctcpMessage->m_command =
+						Kopete::Message::decodeString( KSParser::parse(command), codec );
 				}
 
 				if (space > 0)
-					msg.m_ctcpMessage->m_ctcpRaw = msg.m_ctcpRaw.mid(space).latin1();
+					msg.m_ctcpMessage->m_ctcpRaw =
+						Kopete::Message::decodeString( KSParser::parse(ctcpRaw.mid(space)), codec );
 			}
 
 			msg.m_suffix = Kopete::Message::decodeString( KSParser::parse(suffix), codec );
@@ -324,13 +339,13 @@ bool Message::isValid() const
  * string is splited to get the first part of the message and fill the ctcp command.
  * FIXME: The code currently only match for a textual message or a ctcp message not both mixed as it can be (even if very rare).
  */
-bool Message::extractCtcpCommand(QCString &message, QString &ctcpline, const QTextCodec *codec)
+bool Message::extractCtcpCommand(QCString &message, QCString &ctcpline)
 {
 	uint len = message.length();
 
 	if( message[0] == 1 && message[len-1] == 1 )
 	{
-		ctcpline = ctcpUnquote( unquote( Kopete::Message::decodeString(KSParser::parse(message.mid(1,len-2)), codec) ) );
+		ctcpline = message.mid(1,len-2);
 		message.truncate(0);
 
 		return true;
@@ -353,3 +368,5 @@ void Message::dump() const
 		m_ctcpMessage->dump();
 	}
 }
+
+#include "kircmessage.moc"
