@@ -21,9 +21,11 @@
 #include <kpopupmenu.h>
 
 #include "kopeteawayaction.h"
+#include "kopetemessage.h"
 
 #include "client.h"
 #include "icquserinfo.h"
+#include "oscarsettings.h"
 #include "oscarutils.h"
 
 #include "icqcontact.h"
@@ -49,7 +51,7 @@ void ICQMyselfContact::receivedShortInfo( const QString& contact )
 {
 	if ( Oscar::normalize( contact ) != Oscar::normalize( contactId() ) )
 		return;
-	
+
 	ICQShortInfo shortInfo = static_cast<ICQAccount*>( account() )->engine()->getShortInfo( contact );
 	if ( !shortInfo.nickname.isEmpty() )
 		setProperty( Kopete::Global::Properties::self()->nickName(), shortInfo.nickname );
@@ -66,12 +68,12 @@ ICQAccount::ICQAccount(Kopete::Protocol *parent, QString accountID, const char *
 	kdDebug(14152) << k_funcinfo << accountID << ": Called."<< endl;
 	setMyself( new ICQMyselfContact( this ) );
 	myself()->setOnlineStatus( ICQ::Presence( ICQ::Presence::Offline, ICQ::Presence::Visible ).toOnlineStatus() );
-	
+
 	QString nickName = configGroup()->readEntry("NickName", QString::null);
 	mWebAware = configGroup()->readBoolEntry( "WebAware", false );
 	mHideIP = configGroup()->readBoolEntry( "HideIP", true );
 
-	
+
 	//setIgnoreUnknownContacts(pluginData(protocol(), "IgnoreUnknownContacts").toUInt() == 1);
 
 	/* FIXME: need to do this when web aware or hide ip change
@@ -99,49 +101,22 @@ ICQ::Presence ICQAccount::presence()
 }
 
 
-KAction* ICQAccount::statusAction( const QString &name, ICQ::Presence::Type type, const char *slot )
-{
-	ICQ::Presence pres( type, presence().visibility() );
-	return new KAction( name, pres.toOnlineStatus().iconFor( this ), 0, this, slot, this );
-}
-
-Kopete::AwayAction* ICQAccount::statusActionAway( const QString &name, ICQ::Presence::Type type, const char *slot )
-{
-	ICQ::Presence pres( type, presence().visibility() );
-	return new Kopete::AwayAction( name, pres.toOnlineStatus().iconFor( this ), 0, this, slot, this );
-}
-
 KActionMenu* ICQAccount::actionMenu()
 {
-	// actionMenu is managed by Kopete::Account.  It is deleted when
-	// it is no longer shown, so we can (safely) just make a new one here.
-	KActionMenu* actionMenu = new KActionMenu( accountId(), myself()->onlineStatus().iconFor( this ), this );
-	
-	QString myNick = myself()->property( Kopete::Global::Properties::self()->nickName() ).value().toString();
-	actionMenu->popupMenu()->insertTitle( myself()->onlineStatus().iconFor( myself() ),
-	                                      i18n("%2 <%1>").arg( accountId() ).arg( myNick ) );
-	
-	using namespace ICQ;
-	actionMenu->insert( statusAction( i18n( "O&nline" ), Presence::Online, SLOT( slotGoOnline() ) ) );
-	actionMenu->insert( statusActionAway( i18n( "&Free for Chat" ), Presence::FreeForChat, SLOT( slotGoFFC( const QString & ) ) ) );
-	actionMenu->insert( statusActionAway( i18n( "&Away" ), Presence::Away, SLOT( slotGoAway( const QString & ) ) ) );
-	actionMenu->insert( statusActionAway( i18n( "Not A&vailable" ), Presence::NotAvailable, SLOT( slotGoNA( const QString & ) ) ) );
-	actionMenu->insert( statusActionAway( i18n( "&Do Not Disturb" ), Presence::DoNotDisturb, SLOT( slotGoDND( const QString & ) ) ) );
-	actionMenu->insert( statusActionAway( i18n( "O&ccupied" ), Presence::Occupied, SLOT( slotGoOCC( const QString & ) ) ) );
-	
-	KAction* actionOffline = statusAction( i18n( "O&ffline" ), Presence::Offline, SLOT( slotGoOffline() ) );
-	//actionOffline->setEnabled( isConnected() );
-	actionMenu->insert( actionOffline );
-	
+	KActionMenu* actionMenu = Kopete::Account::actionMenu();
+
 	actionMenu->popupMenu()->insertSeparator();
-	
-	KToggleAction* actionInvisible = new KToggleAction( i18n( "In&visible" ), "icq_invisible", 0, this, SLOT( slotToggleInvisible() ), this );
+
+	KToggleAction* actionInvisible =
+	    new KToggleAction( i18n( "In&visible" ),
+	                       ICQ::Presence( presence().type(), ICQ::Presence::Invisible ).toOnlineStatus().iconFor( this ),
+	                       0, this, SLOT( slotToggleInvisible() ), this );
 	actionInvisible->setChecked( presence().visibility() == ICQ::Presence::Invisible );
 	actionMenu->insert( actionInvisible );
-	
+
 	//actionMenu->popupMenu()->insertSeparator();
 	//actionMenu->insert( new KToggleAction( i18n( "Send &SMS..." ), 0, 0, this, SLOT( slotSendSMS() ), this, "ICQAccount::mActionSendSMS") );
-	
+
 	return actionMenu;
 }
 
@@ -150,13 +125,18 @@ void ICQAccount::connectWithPassword( const QString &password )
 {
 	if ( password.isNull() )
 		return;
-	
+
 	kdDebug(14153) << k_funcinfo << "accountId='" << accountId() << "'" << endl;
-	
-	ICQ::Presence pres = ICQ::Presence::fromOnlineStatus( initialStatus() );
+
+	Kopete::OnlineStatus status = initialStatus();
+	if ( status == Kopete::OnlineStatus() &&
+	     status.status() == Kopete::OnlineStatus::Unknown )
+		//use default online in case of invalid online status for connecting
+		status = Kopete::OnlineStatus( Kopete::OnlineStatus::Online );
+	ICQ::Presence pres = ICQ::Presence::fromOnlineStatus( status );
 	bool accountIsOffline = ( presence().type() == ICQ::Presence::Offline ||
 	                          myself()->onlineStatus() == protocol()->statusManager()->connectingStatus() );
-	
+
 	if ( accountIsOffline )
 	{
 		myself()->setOnlineStatus( protocol()->statusManager()->connectingStatus() );
@@ -165,20 +145,26 @@ void ICQAccount::connectWithPassword( const QString &password )
 		QString server = configGroup()->readEntry( "Server", QString::fromLatin1( "login.oscar.aol.com" ) );
 		uint port = configGroup()->readNumEntry( "Port", 5190 );
 		Connection* c = setupConnection( server, port );
-		
+
+		//set up the settings for the account
+		Oscar::Settings* oscarSettings = engine()->clientSettings();
+		oscarSettings->setWebAware( configGroup()->readBoolEntry( "WebAware", false ) );
+		oscarSettings->setHideIP( configGroup()->readBoolEntry( "HideIP", true ) );
+		oscarSettings->setRequireAuth( configGroup()->readBoolEntry( "RequireAuth", false ) );
+		oscarSettings->setRespectRequireAuth( configGroup()->readBoolEntry( "RespectRequireAuth", true ) );
 		//FIXME: also needed for the other call to setStatus (in setPresenceTarget)
 		DWORD status = pres.toOscarStatus();
-		
+
 		if ( !mHideIP )
 			status |= ICQ::StatusCode::SHOWIP;
 		if ( mWebAware )
 			status |= ICQ::StatusCode::WEBAWARE;
-		
+
 		engine()->setIsIcq( true );
 		engine()->setStatus( status );
 		engine()->start( server, port, accountId(), password );
 		engine()->connectToServer( c, server, true /* doAuth */ );
-		
+
 	}
 }
 
@@ -191,41 +177,6 @@ void ICQAccount::disconnected( DisconnectReason reason )
 }
 
 
-void ICQAccount::slotGoOnline()
-{
-	setPresenceType( ICQ::Presence::Online );
-}
-
-void ICQAccount::slotGoFFC( const QString &reason )
-{
-	setPresenceType( ICQ::Presence::FreeForChat, reason );
-}
-
-void ICQAccount::slotGoAway( const QString &reason )
-{
-	setPresenceType( ICQ::Presence::Away, reason );
-}
-
-void ICQAccount::slotGoNA( const QString &reason )
-{
-	setPresenceType( ICQ::Presence::NotAvailable, reason );
-}
-
-void ICQAccount::slotGoOCC( const QString &reason )
-{
-	setPresenceType( ICQ::Presence::Occupied, reason );
-}
-
-void ICQAccount::slotGoDND( const QString &reason )
-{
-	setPresenceType( ICQ::Presence::DoNotDisturb, reason );
-}
-
-void ICQAccount::slotGoOffline()
-{
-	setPresenceType( ICQ::Presence::Offline );
-}
-
 void ICQAccount::slotToggleInvisible()
 {
 	using namespace ICQ;
@@ -236,9 +187,9 @@ void ICQAccount::setAway( bool away, const QString &awayReason )
 {
 	kdDebug(14153) << k_funcinfo << "account='" << accountId() << "'" << endl;
 	if ( away )
-		slotGoAway( awayReason );
+		setPresenceType( ICQ::Presence::Away, awayReason );
 	else
-		slotGoOnline();
+		setPresenceType( ICQ::Presence::Online );
 }
 
 
@@ -247,7 +198,7 @@ void ICQAccount::setInvisible( ICQ::Presence::Visibility vis )
 	ICQ::Presence pres = presence();
 	if ( vis == pres.visibility() )
 		return;
-	
+
 	kdDebug(14153) << k_funcinfo << "changing invisible setting to " << (int)vis << endl;
 	setPresenceTarget( ICQ::Presence( pres.type(), vis ) );
 }
@@ -258,15 +209,16 @@ void ICQAccount::setPresenceType( ICQ::Presence::Type type, const QString &messa
 	ICQ::Presence pres = presence();
 	kdDebug(14153) << k_funcinfo << "new type=" << (int)type << ", old type=" << (int)pres.type() << endl;
 	//setAwayMessage(awayMessage);
-	setPresenceTarget( ICQ::Presence( type, pres.visibility() ) );
+	setPresenceTarget( ICQ::Presence( type, pres.visibility() ), message );
+	myself()->setProperty( Kopete::Global::Properties::self()->awayMessage(), message );
 }
 
-void ICQAccount::setPresenceTarget( const ICQ::Presence &newPres )
+void ICQAccount::setPresenceTarget( const ICQ::Presence &newPres, const QString &message )
 {
 	bool targetIsOffline = (newPres.type() == ICQ::Presence::Offline);
 	bool accountIsOffline = ( presence().type() == ICQ::Presence::Offline ||
 	                          myself()->onlineStatus() == protocol()->statusManager()->connectingStatus() );
-	
+
 	if ( targetIsOffline )
 	{
 		OscarAccount::disconnect();
@@ -275,11 +227,38 @@ void ICQAccount::setPresenceTarget( const ICQ::Presence &newPres )
 	}
 	else if ( accountIsOffline )
 	{
+		// set status message if given
+		if ( ! message.isEmpty() )
+			engine()->setStatusMessage( message );
 		OscarAccount::connect( newPres.toOnlineStatus() );
 	}
 	else
 	{
-		engine()->setStatus( newPres.toOscarStatus() );
+		engine()->setStatus( newPres.toOscarStatus(), message );
+	}
+}
+
+
+void ICQAccount::setOnlineStatus( const Kopete::OnlineStatus& status, const QString& reason )
+{
+	if ( status.status() == Kopete::OnlineStatus::Invisible )
+	{
+		// called from outside, i.e. not by our custom action menu entry...
+
+		if ( presence().type() == ICQ::Presence::Offline )
+		{
+			// ...when we are offline go online invisible.
+			setPresenceTarget( ICQ::Presence( ICQ::Presence::Online, ICQ::Presence::Invisible ) );
+		}
+		else
+		{
+			// ...when we are not offline set invisible.
+			setInvisible( ICQ::Presence::Invisible );
+		}
+	}
+	else
+	{
+		setPresenceType( ICQ::Presence::fromOnlineStatus( status ).type(), reason );
 	}
 }
 
@@ -289,23 +268,16 @@ OscarContact *ICQAccount::createNewContact( const QString &contactId, Kopete::Me
 	ICQContact* contact = new ICQContact( this, contactId, parentContact, QString::null, ssiItem );
 	if ( !ssiItem.alias().isEmpty() )
 		contact->setProperty( Kopete::Global::Properties::self()->nickName(), ssiItem.alias() );
-	
+
 	if ( isConnected() )
 		contact->loggedIn();
-	
+
 	return contact;
 }
 
-QString ICQAccount::sanitizedMessage( const Oscar::Message& message )
+QString ICQAccount::sanitizedMessage( const QString& message )
 {
-	if ( message.type() == 1 || message.type() == 4 )
-	{
-		return message.text();
-	}
-	else 
-		kdWarning(OSCAR_RAW_DEBUG) << k_funcinfo << "ICQ type 2 messages not supported yet. Message text:" << message.text() << endl;
-	
-	return QString::null;
+	return Kopete::Message::escape( message );
 }
 
 
