@@ -28,7 +28,7 @@
 #include <kdeversion.h>
 
 #include "kopeteaccount.h"
-#include "kopetemessagemanagerfactory.h"
+#include "kopetechatsessionmanager.h"
 #include "kopetemetacontact.h"
 #include "kopetecontactlist.h"
 #include "kopetegroup.h"
@@ -70,7 +70,10 @@ void OscarContact::serialize(QMap<QString, QString> &serializedData,
 
 bool OscarContact::isOnServer() const
 {
-	return ( m_ssiItem.type() != 0xFFFF );
+    SSIManager* serverList = mAccount->engine()->ssiManager();
+    SSI ssi = serverList->findContact( Oscar::normalize( contactId() ) );
+
+	return ( ssi && ssi.type() != 0xFFFF );
 }
 
 void OscarContact::setSSIItem( const Oscar::SSI& ssiItem )
@@ -104,7 +107,7 @@ Kopete::ChatSession* OscarContact::manager( CanCreateFlags canCreate )
 		connect(mMsgManager, SIGNAL( destroyed() ),
 		        this, SLOT( chatSessionDestroyed() ) );
 
-		connect(mMsgManager, SIGNAL( typingMsg( bool ) ),
+		connect(mMsgManager, SIGNAL( myselfTyping( bool ) ),
 		        this, SLOT( slotTyping( bool ) ) );
 	}
 	return mMsgManager;
@@ -172,33 +175,66 @@ void OscarContact::sync(unsigned int flags)
 void OscarContact::userInfoUpdated( const QString& contact, const UserDetails& details  )
 {
 	Q_UNUSED( contact );
+	setProperty( Kopete::Global::Properties::self()->onlineSince(), details.onlineSinceTime() );
 	setIdleTime( details.idleTime() );
 	m_warningLevel = details.warningLevel();
+	m_details = details;
+	
+	QStringList capList;
+	// Append client name and version in case we found one
+	if ( details.userClass() & 0x0080 /* WIRELESS */ )
+		capList << i18n( "Mobile AIM Client" );
+	else
+	{
+		if ( !details.clientName().isEmpty() )
+		{
+			capList << i18n( "Translators: client name and version",
+			                "%1").arg( details.clientName() );
+		}
+	}
+	
+	// and now for some general informative capabilities
+	if ( details.hasCap( CAP_BUDDYICON ) )
+		capList << i18n( "Buddy icons" );
+	if ( details.hasCap( CAP_UTF8 ) )
+		capList << i18n( "UTF-8" );
+	if ( details.hasCap( CAP_RTFMSGS ) )
+		capList << i18n( "Rich text messages" );
+	if ( details.hasCap( CAP_CHAT ) )
+		capList << i18n( "Group chat" );
+	if ( details.hasCap( CAP_VOICE ) )
+		capList << i18n( "Voice chat" );
+	if ( details.hasCap( CAP_IMIMAGE ) )
+		capList << i18n( "DirectIM/IMImage" );
+	if ( details.hasCap( CAP_SENDBUDDYLIST ) )
+		capList << i18n( "Send buddy list" );
+	if ( details.hasCap( CAP_SENDFILE ) )
+		capList << i18n( "File transfers" );
+	if ( details.hasCap( CAP_GAMES ) || details.hasCap( CAP_GAMES2 ) )
+		capList << i18n( "Games" );
+	if ( details.hasCap( CAP_TRILLIAN ) )
+		capList << i18n( "Trillian user" );
+	
+	m_clientFeatures = capList.join( ", " );
+	emit featuresUpdated();
 }
 
-void OscarContact::slotSendMsg( Kopete::Message& msg, Kopete::ChatSession* session )
+void OscarContact::startedTyping()
 {
-	//Why is this unused?
-	Q_UNUSED( session );
-	Oscar::Message message;
+	if ( mMsgManager )
+		mMsgManager->receivedTypingMsg( this, true );
+}
 
-	message.setText( msg.plainBody() );
-	
-	message.setTimestamp( msg.timestamp() );
-	message.setSender( mAccount->accountId() );
-	message.setReceiver( mName );
-	message.setType( 0x01 );
-	
-	//TODO add support for type 2 messages
-	/*if ( msg.type() == Kopete::Message::PlainText )
-		message.setType( 0x01 );
-	else
-		message.setType( 0x02 );*/
-	//TODO: we need to check for channel 0x04 messages too;
-	
-	mAccount->engine()->sendMessage( message );
-	manager(Kopete::Contact::CanCreate)->appendMessage(msg);
-	manager(Kopete::Contact::CanCreate)->messageSucceeded();
+void OscarContact::stoppedTyping()
+{
+	if ( mMsgManager )
+		mMsgManager->receivedTypingMsg( this, false );
+}
+
+void OscarContact::slotTyping( bool typing )
+{
+	if ( this != account()->myself() )
+		account()->engine()->sendTyping( contactId(), typing );
 }
 
 #include "oscarcontact.moc"
