@@ -3,10 +3,10 @@
  *
  * libyahoo2 is available at http://libyahoo2.sourceforge.net/
  *
- * $Revision: 1.7 $
- * $Date: 2004/12/31 18:02:39 $
+ * $Revision: 421285 $
+ * $Date: 2005-06-02 19:29:53 +0200 (Do, 02 Jun 2005) $
  * 
- * Copyright (C) 2002, Philip S Tellis <philip . tellis AT gmx . net>
+ * Copyright (C) 2002-2004, Philip S Tellis <philip.tellis AT gmx.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -136,7 +136,7 @@ char * yahoo_status_code(enum yahoo_status s)
 
 #define YAHOO_DEBUGLOG ext_yahoo_log
 
-int ext_yahoo_log(char *fmt,...)
+int ext_yahoo_log(const char *fmt,...)
 {
 	va_list ap;
 
@@ -303,13 +303,14 @@ static int yahoo_webcam_timeout_callback(int id)
 YList * conferences = NULL;
 typedef struct {
 	int id;
+	char * me;
 	char * room_name;
 	char * host;
 	YList * members;
 	int joined;
 } conf_room;
 
-static char * get_buddy_name(char * yid)
+static const char * get_buddy_name(const char * yid)
 {
 	YList * b;
 	for (b = buddies; b; b = b->next) {
@@ -321,12 +322,12 @@ static char * get_buddy_name(char * yid)
 	return yid;
 }
 
-static conf_room * find_conf_room_by_name_and_id(int id, const char * name)
+static conf_room * find_conf_room_by_name_and_id(int id, const char * me, const char * name)
 {
 	YList * l;
 	for(l = conferences; l; l=l->next) {
 		conf_room * cr = l->data;
-		if(cr->id == id && !strcmp(name, cr->room_name)) {
+		if(cr->id == id && !strcmp(name, cr->room_name) && (me == NULL || cr->me == NULL || !strcmp(me, cr->me))) {
 			return cr;
 		}
 	}
@@ -334,28 +335,30 @@ static conf_room * find_conf_room_by_name_and_id(int id, const char * name)
 	return NULL;
 }
 
-void ext_yahoo_got_conf_invite(int id, char *who, char *room, char *msg, YList *members)
+void ext_yahoo_got_conf_invite(int id, const char *me, const char *who, const char *room, const char *msg, YList *members)
 {
 	conf_room * cr = y_new0(conf_room, 1);
 	cr->room_name = strdup(room);
+	cr->me = strdup(me);
 	cr->host = strdup(who);
 	cr->members = members;
 	cr->id = id;
 
 	conferences = y_list_append(conferences, cr);
 
-	print_message(("%s has invited you to a conference: %s\n"
+	print_message(("%s has invited you [%s] to a conference: %s\n"
 				"with the message: %s",
-				who, room, msg));
+				who, me, room, msg));
 
 	for(; members; members=members->next)
 		print_message(("\t%s", (char *)members->data));
 	
 }
-void ext_yahoo_conf_userdecline(int id, char *who, char *room, char *msg)
+void ext_yahoo_conf_userdecline(int id, const char *me, const char *who, const char *room, const char *msg)
 {
 	YList * l;
-	conf_room * cr = find_conf_room_by_name_and_id(id, room);
+	/* TODO: probably have to use the me arg to find the room */
+	conf_room * cr = find_conf_room_by_name_and_id(id, me, room);
 	
 	if(cr)
 	for(l = cr->members; l; l=l->next) {
@@ -368,12 +371,12 @@ void ext_yahoo_conf_userdecline(int id, char *who, char *room, char *msg)
 		}
 	}
 
-	print_message(("%s declined the invitation to %s\n"
-				"with the message: %s", who, room, msg));
+	print_message(("%s declined the invitation from %s to %s\n"
+				"with the message: %s", who, me, room, msg));
 }
-void ext_yahoo_conf_userjoin(int id, char *who, char *room)
+void ext_yahoo_conf_userjoin(int id, const char *me, const char *who, const char *room)
 {
-	conf_room * cr = find_conf_room_by_name_and_id(id, room);
+	conf_room * cr = find_conf_room_by_name_and_id(id, me, room);
 	if(cr) {
 	YList * l = NULL;
 	for(l = cr->members; l; l=l->next) {
@@ -385,13 +388,13 @@ void ext_yahoo_conf_userjoin(int id, char *who, char *room)
 		cr->members = y_list_append(cr->members, strdup(who));
 	}
 
-	print_message(("%s joined the conference %s", who, room));
+	print_message(("%s joined the conference %s [%s]", who, room, me));
 
 }
-void ext_yahoo_conf_userleave(int id, char *who, char *room)
+void ext_yahoo_conf_userleave(int id, const char *me, const char *who, const char *room)
 {
 	YList * l;
-	conf_room * cr = find_conf_room_by_name_and_id(id, room);
+	conf_room * cr = find_conf_room_by_name_and_id(id, me, room);
 	
 	if(cr)
 	for(l = cr->members; l; l=l->next) {
@@ -404,18 +407,18 @@ void ext_yahoo_conf_userleave(int id, char *who, char *room)
 		}
 	}
 
-	print_message(("%s left the conference %s", who, room));
+	print_message(("%s left the conference %s [%s]", who, room, me));
 }
-void ext_yahoo_conf_message(int id, char *who, char *room, char *msg, int utf8)
+void ext_yahoo_conf_message(int id, const char *me, const char *who, const char *room, const char *msg, int utf8)
 {
-	char * umsg = msg;
+	char * umsg = (char *)msg;
 
 	if(utf8)
 		umsg = y_utf8_to_str(msg);
 
 	who = get_buddy_name(who);
 
-	print_message(("%s (in %s): %s", who, room, umsg));
+	print_message(("%s (in %s [%s]): %s", who, room, me, umsg));
 
 	if(utf8)
 		FREE(umsg);
@@ -439,14 +442,14 @@ static void print_chat_member(struct yahoo_chat_member *ycm)
 	printf("  Location: %s", ycm->location);
 }
 
-void ext_yahoo_chat_cat_xml(int id, char *xml) 
+void ext_yahoo_chat_cat_xml(int id, const char *xml) 
 {
 	print_message(("%s", xml));
 }
 
-void ext_yahoo_chat_join(int id, char *room, char * topic, YList *members)
+void ext_yahoo_chat_join(int id, const char *me, const char *room, const char * topic, YList *members, int fd)
 {
-	print_message(("You have joined the chatroom %s with topic %s", room, topic));
+	print_message(("You [%s] have joined the chatroom %s with topic %s", me, room, topic));
 
 	while(members) {
 		YList *n = members->next;
@@ -462,22 +465,22 @@ void ext_yahoo_chat_join(int id, char *room, char * topic, YList *members)
 		members=n;
 	}
 }
-void ext_yahoo_chat_userjoin(int id, char *room, struct yahoo_chat_member *who)
+void ext_yahoo_chat_userjoin(int id, const char *me, const char *room, struct yahoo_chat_member *who)
 {
 	print_chat_member(who);
-	print_message((" joined the chatroom %s", room));
+	print_message((" joined the chatroom %s [%s]", room, me));
 	FREE(who->id);
 	FREE(who->alias);
 	FREE(who->location);
 	FREE(who);
 }
-void ext_yahoo_chat_userleave(int id, char *room, char *who)
+void ext_yahoo_chat_userleave(int id, const char *me, const char *room, const char *who)
 {
-	print_message(("%s left the chatroom %s", who, room));
+	print_message(("%s left the chatroom %s [%s]", who, room, me));
 }
-void ext_yahoo_chat_message(int id, char *who, char *room, char *msg, int msgtype, int utf8)
+void ext_yahoo_chat_message(int id, const char *me, const char *who, const char *room, const char *msg, int msgtype, int utf8)
 {
-	char * umsg = msg;
+	char * umsg = (char *)msg;
 	char * charpos;
 
 	if(utf8)
@@ -492,19 +495,21 @@ void ext_yahoo_chat_message(int id, char *who, char *room, char *msg, int msgtyp
 	}
 
 	if (msgtype == 2) {
-		print_message(("(in %s) %s %s", room, who, umsg));
+		print_message(("(in %s [%s]) %s %s", room, me, who, umsg));
 	} else {
-		print_message(("(in %s) %s: %s", room, who, umsg));
+		print_message(("(in %s [%s]) %s: %s", room, me, who, umsg));
 	}
 
 	if(utf8)
 		FREE(umsg);
 }
 
-void ext_yahoo_status_changed(int id, char *who, int stat, char *msg, int away)
+void ext_yahoo_status_changed(int id, const char *who, int stat, const char *msg, int away, int idle, int mobile)
 {
 	yahoo_account * ya=NULL;
 	YList * b;
+	char buf[1024];
+
 	for(b = buddies; b; b = b->next) {
 		if(!strcmp(((yahoo_account *)b->data)->yahoo_id, who)) {
 			ya = b->data;
@@ -512,14 +517,34 @@ void ext_yahoo_status_changed(int id, char *who, int stat, char *msg, int away)
 		}
 	}
 	
-	if(msg)
-		print_message(("%s (%s) is now %s", ya?ya->name:who, who, msg))
-	else if(stat == YAHOO_STATUS_IDLE)
-		print_message(("%s (%s) idle for %d:%02d:%02d", ya?ya->name:who, who, 
-					away/3600, (away/60)%60, away%60))
-	else
-		print_message(("%s (%s) is now %s", ya?ya->name:who, who, 
-					yahoo_status_code(stat)))
+	if (msg == NULL) {
+		sprintf(buf, "%s", yahoo_status_code(stat));
+	}
+	else if (stat == YAHOO_STATUS_CUSTOM) {
+		sprintf(buf, "%s", msg);
+	} else {
+		sprintf(buf, "%s: %s", yahoo_status_code(stat), msg);
+	}
+
+	if (away > 0) {
+		char away_buf[32];
+		sprintf(away_buf, " away[%d]", away);
+		strcat(buf, away_buf);
+	}
+
+	if (mobile > 0) {
+		char mobile_buf[32];
+		sprintf(mobile_buf, " mobile[%d]", mobile);
+		strcat(buf, mobile_buf);
+	}
+
+	if (idle > 0) {
+		char time_buf[32];
+		sprintf(time_buf, " idle for %d:%02d:%02d", idle/3600, (idle/60)%60, idle%60);
+		strcat(buf, time_buf);
+	}
+
+	print_message(("%s (%s) is now %s", ya?ya->name:who, who, buf))
 
 	if(ya) {
 		ya->status = stat;
@@ -557,12 +582,12 @@ void ext_yahoo_got_ignore(int id, YList * igns)
 {
 }
 
-void ext_yahoo_got_im(int id, char *who, char *msg, long tm, int stat, int utf8)
+void ext_yahoo_got_im(int id, const char *me, const char *who, const char *msg, long tm, int stat, int utf8)
 {
-	char *umsg = msg;
+	char *umsg = (char *)msg;
 
 	if(stat == 2) {
-		LOG(("Error sending message to %s", who));
+		LOG(("Error sending message from %s to %s", me, who));
 		return;
 	}
 
@@ -577,29 +602,29 @@ void ext_yahoo_got_im(int id, char *who, char *msg, long tm, int stat, int utf8)
 	if(tm) {
 		char timestr[255];
 
-		strncpy(timestr, ctime(&tm), sizeof(timestr));
+		strncpy(timestr, ctime((time_t *)&tm), sizeof(timestr));
 		timestr[strlen(timestr) - 1] = '\0';
 
-		print_message(("[Offline message at %s from %s]: %s", 
-				timestr, who, umsg))
+		print_message(("[Offline message at %s to %s from %s]: %s", 
+				timestr, me, who, umsg))
 	} else {
 		if(!strcmp(umsg, "<ding>")) 
 			printf("\a");
-		print_message(("%s: %s", who, umsg))
+		print_message(("[%s]%s: %s", me, who, umsg))
 	}
 
 	if(utf8)
 		FREE(umsg);
 }
 
-void ext_yahoo_rejected(int id, char *who, char *msg)
+void ext_yahoo_rejected(int id, const char *who, const char *msg)
 {
 	print_message(("%s has rejected you%s%s", who, 
 				(msg?" with the message:\n":"."), 
 				(msg?msg:"")));
 }
 
-void ext_yahoo_contact_added(int id, char *myid, char *who, char *msg)
+void ext_yahoo_contact_added(int id, const char *myid, const char *who, const char *msg)
 {
 	char buff[1024];
 
@@ -620,17 +645,17 @@ void ext_yahoo_contact_added(int id, char *myid, char *who, char *msg)
 */
 }
 
-void ext_yahoo_typing_notify(int id, char *who, int stat)
+void ext_yahoo_typing_notify(int id, const char* me, const char *who, int stat)
 {
 	if(stat && do_typing_notify)
-		print_message(("%s is typing...", who));
+		print_message(("[%s]%s is typing...", me, who));
 }
 
-void ext_yahoo_game_notify(int id, char *who, int stat)
+void ext_yahoo_game_notify(int id, const char *me, const char *who, int stat)
 {
 }
 
-void ext_yahoo_mail_notify(int id, char *from, char *subj, int cnt)
+void ext_yahoo_mail_notify(int id, const char *from, const char *subj, int cnt)
 {
 	char buff[1024] = {0};
 	
@@ -654,7 +679,7 @@ void ext_yahoo_mail_notify(int id, char *from, char *subj, int cnt)
 }
 
 void ext_yahoo_got_webcam_image(int id, const char *who,
-		unsigned char *image, unsigned int image_size, unsigned int real_size,
+		const unsigned char *image, unsigned int image_size, unsigned int real_size,
 		unsigned int timestamp)
 {
 	static unsigned char *cur_image = NULL;
@@ -694,7 +719,7 @@ void ext_yahoo_got_webcam_image(int id, const char *who,
 	}
 }
 
-void ext_yahoo_webcam_viewer(int id, char *who, int connect)
+void ext_yahoo_webcam_viewer(int id, const char *who, int connect)
 {
 	switch (connect)
 	{
@@ -711,7 +736,7 @@ void ext_yahoo_webcam_viewer(int id, char *who, int connect)
 	}
 }
 
-void ext_yahoo_webcam_closed(int id, char *who, int reason)
+void ext_yahoo_webcam_closed(int id, const char *who, int reason)
 {
 	switch(reason)
 	{
@@ -744,21 +769,21 @@ void ext_yahoo_webcam_data_request(int id, int send)
 	send_webcam_images = send;
 }
 
-void ext_yahoo_webcam_invite(int id, char *from)
+void ext_yahoo_webcam_invite(int id, const char *me, const char *from)
 {
-	print_message(("Got a webcam invitation from %s", from));
+	print_message(("Got a webcam invitation to %s from %s", me, from));
 }
 
-void ext_yahoo_webcam_invite_reply(int id, char *from, int accept)
+void ext_yahoo_webcam_invite_reply(int id, const char *me, const char *from, int accept)
 {
 	if(accept) {
-		print_message(("%s accepted webcam invitation...", from));
+		print_message(("[%s]%s accepted webcam invitation...", me, from));
 	} else {
-		print_message(("%s declined webcam invitation...", from));
+		print_message(("[%s]%s declined webcam invitation...", me, from));
 	}
 }
 
-void ext_yahoo_system_message(int id, char *msg)
+void ext_yahoo_system_message(int id, const char *msg)
 {
 	if(ignore_system)
 		return;
@@ -779,6 +804,7 @@ void yahoo_logout()
 		conf_room * cr = conferences->data;
 		if(cr->joined)
 			yahoo_conference_logoff(ylad->id, NULL, cr->members, cr->room_name);
+		FREE(cr->me);
 		FREE(cr->room_name);
 		FREE(cr->host);
 		while(cr->members) {
@@ -809,6 +835,7 @@ void ext_yahoo_login(yahoo_local_account * ylad, int login_mode)
 
 	ylad->id = yahoo_init_with_attributes(ylad->yahoo_id, ylad->password, 
 			"local_host", local_host,
+			"pager_port", 23,
 			NULL);
 	ylad->status = YAHOO_STATUS_OFFLINE;
 	yahoo_login(ylad->id, login_mode);
@@ -826,7 +853,7 @@ void ext_yahoo_got_cookies(int id)
 	/*yahoo_get_yab(id);*/
 }
 
-void ext_yahoo_login_response(int id, int succ, char *url)
+void ext_yahoo_login_response(int id, int succ, const char *url)
 {
 	char buff[1024];
 
@@ -835,16 +862,25 @@ void ext_yahoo_login_response(int id, int succ, char *url)
 		print_message(("logged in"));
 		return;
 		
+	} else if(succ == YAHOO_LOGIN_UNAME) {
+
+		snprintf(buff, sizeof(buff), "Could not log into Yahoo service - username not recognised.  Please verify that your username is correctly typed.");
 	} else if(succ == YAHOO_LOGIN_PASSWD) {
 
-		snprintf(buff, sizeof(buff), "Could not log into Yahoo service.  Please verify that your username and password are correctly typed.");
+		snprintf(buff, sizeof(buff), "Could not log into Yahoo service - password incorrect.  Please verify that your password is correctly typed.");
 
 	} else if(succ == YAHOO_LOGIN_LOCK) {
 		
 		snprintf(buff, sizeof(buff), "Could not log into Yahoo service.  Your account has been locked.\nVisit %s to reactivate it.", url);
 
 	} else if(succ == YAHOO_LOGIN_DUPL) {
+
 		snprintf(buff, sizeof(buff), "You have been logged out of the yahoo service, possibly due to a duplicate login.");
+	} else if(succ == YAHOO_LOGIN_SOCK) {
+
+		snprintf(buff, sizeof(buff), "The server closed the socket.");
+	} else {
+		snprintf(buff, sizeof(buff), "Could not log in, unknown reason: %d.", succ);
 	}
 
 	ylad->status = YAHOO_STATUS_OFFLINE;
@@ -853,12 +889,17 @@ void ext_yahoo_login_response(int id, int succ, char *url)
 	poll_loop=0;
 }
 
-void ext_yahoo_error(int id, char *err, int fatal)
+void ext_yahoo_error(int id, const char *err, int fatal, int num)
 {
 	fprintf(stdout, "Yahoo Error: ");
 	fprintf(stdout, "%s", err);
-	/*
 	switch(num) {
+		case E_UNKNOWN:
+			fprintf(stdout, "unknown error %s", err);
+			break;
+		case E_CUSTOM:
+			fprintf(stdout, "custom error %s", err);
+			break;
 		case E_CONFNOTAVAIL:
 			fprintf(stdout, "%s is not available for the conference", err);
 			break;
@@ -871,8 +912,13 @@ void ext_yahoo_error(int id, char *err, int fatal)
 		case E_IGNORECONF:
 			fprintf(stdout, "%s is in buddy list - cannot ignore ", err);
 			break;
+		case E_SYSTEM:
+			fprintf(stdout, "system error %s", err);
+			break;
+		case E_CONNECTION:
+			fprintf(stdout, "server connection error %s", err);
+			break;
 	}
-	*/
 	fprintf(stdout, "\n");
 	if(fatal)
 		yahoo_logout();
@@ -898,7 +944,7 @@ void yahoo_set_current_state(int yahoo_state)
 		yahoo_set_away(ylad->id, yahoo_state, NULL, 1);
 }
 
-int ext_yahoo_connect(char *host, int port)
+int ext_yahoo_connect(const char *host, int port)
 {
 	struct sockaddr_in serv_addr;
 	static struct hostent *server;
@@ -954,35 +1000,40 @@ int ext_yahoo_connect(char *host, int port)
  */
 YList *connections = NULL;
 struct _conn {
-	int id;
+	int tag;
 	int fd;
+	int id;
 	yahoo_input_condition cond;
 	void *data;
 	int remove;
 };
+static int connection_tags=0;
 
-void ext_yahoo_add_handler(int id, int fd, yahoo_input_condition cond, void *data)
+int ext_yahoo_add_handler(int id, int fd, yahoo_input_condition cond, void *data)
 {
 	struct _conn *c = y_new0(struct _conn, 1);
+	c->tag = ++connection_tags;
 	c->id = id;
 	c->fd = fd;
 	c->cond = cond;
 	c->data = data;
 
-	LOG(("Add %d for %d", fd, id));
+	LOG(("Add %d for %d, tag %d", fd, id, c->tag));
 
 	connections = y_list_prepend(connections, c);
+
+	return c->tag;
 }
 
-void ext_yahoo_remove_handler(int id, int fd)
+void ext_yahoo_remove_handler(int id, int tag)
 {
 	YList *l;
 	for(l = connections; l; l = y_list_next(l)) {
 		struct _conn *c = l->data;
-		if(c->fd == fd) {
+		if(c->tag == tag) {
 			/* don't actually remove it, just mark it for removal */
 			/* we'll remove when we start the next poll cycle */
-			LOG(("Marking id:%d fd:%d for removal", id, fd));
+			LOG(("Marking id:%d fd:%d tag:%d for removal", c->id, c->fd, c->tag));
 			c->remove = 1;
 			return;
 		}
@@ -993,6 +1044,7 @@ struct connect_callback_data {
 	yahoo_connect_callback callback;
 	void * callback_data;
 	int id;
+	int tag;
 };
 
 static void connect_complete(void *data, int source, yahoo_input_condition condition)
@@ -1000,7 +1052,7 @@ static void connect_complete(void *data, int source, yahoo_input_condition condi
 	struct connect_callback_data *ccd = data;
 	int error, err_size = sizeof(error);
 
-	ext_yahoo_remove_handler(-1, source);
+	ext_yahoo_remove_handler(0, ccd->tag);
 	getsockopt(source, SOL_SOCKET, SO_ERROR, &error, (socklen_t *)&err_size);
 
 	if(error) {
@@ -1024,7 +1076,7 @@ void yahoo_callback(struct _conn *c, yahoo_input_condition cond)
 	} else {
 		if(cond & YAHOO_INPUT_READ)
 			ret = yahoo_read_ready(c->id, c->fd, c->data);
-		if(cond & YAHOO_INPUT_WRITE)
+		if(ret>0 && cond & YAHOO_INPUT_WRITE)
 			ret = yahoo_write_ready(c->id, c->fd, c->data);
 
 		if(ret == -1)
@@ -1039,7 +1091,7 @@ void yahoo_callback(struct _conn *c, yahoo_input_condition cond)
 	}
 }
 
-int ext_yahoo_connect_async(int id, char *host, int port, 
+int ext_yahoo_connect_async(int id, const char *host, int port, 
 		yahoo_connect_callback callback, void *data)
 {
 	struct sockaddr_in serv_addr;
@@ -1074,9 +1126,12 @@ int ext_yahoo_connect_async(int id, char *host, int port,
 		ccd->callback_data = data;
 		ccd->id = id;
 
-		ext_yahoo_add_handler(-1, servfd, YAHOO_INPUT_WRITE, ccd);
-		return 1;
+		ccd->tag = ext_yahoo_add_handler(-1, servfd, YAHOO_INPUT_WRITE, ccd);
+		return ccd->tag;
 	} else {
+		if(error == -1) {
+			LOG(("%s", strerror(errno)));
+		}
 		close(servfd);
 		return -1;
 	}
@@ -1131,7 +1186,7 @@ static void process_commands(char *line)
 			copy = tmp+1;
 		}
 		msg = copy;
-		cr = find_conf_room_by_name_and_id(ylad->id, to);
+		cr = find_conf_room_by_name_and_id(ylad->id, NULL, to);
 		if(!cr) {
 			print_message(("no such room: %s", copy));
 			goto end_parse;
@@ -1141,7 +1196,7 @@ static void process_commands(char *line)
 	} else if(!strncasecmp(cmd, "CLS", strlen("CLS"))) {
 		YList * l;
 		if(copy) {
-			conf_room * cr = find_conf_room_by_name_and_id(ylad->id, copy);
+			conf_room * cr = find_conf_room_by_name_and_id(ylad->id, NULL, copy);
 			if(!cr) {
 				print_message(("no such room: %s", copy));
 				goto end_parse;
@@ -1191,7 +1246,7 @@ static void process_commands(char *line)
 			l = y_list_append(l, copy);
 		}
 
-		cr = find_conf_room_by_name_and_id(ylad->id, room);
+		cr = find_conf_room_by_name_and_id(ylad->id, NULL, room);
 		if(!cr) {
 			print_message(("no such room: %s", room));
 			y_list_free(l);
@@ -1206,7 +1261,7 @@ static void process_commands(char *line)
 		y_list_free(l);
 
 	} else if(!strncasecmp(cmd, "CLN", strlen("CLN"))) {
-		conf_room * cr = find_conf_room_by_name_and_id(ylad->id, copy);
+		conf_room * cr = find_conf_room_by_name_and_id(ylad->id, NULL, copy);
 		YList * l;
 		if(!cr) {
 			print_message(("no such room: %s", copy));
@@ -1224,7 +1279,7 @@ static void process_commands(char *line)
 		yahoo_conference_logon(ylad->id, NULL, cr->members, copy);
 
 	} else if(!strncasecmp(cmd, "CLF", strlen("CLF"))) {
-		conf_room * cr = find_conf_room_by_name_and_id(ylad->id, copy);
+		conf_room * cr = find_conf_room_by_name_and_id(ylad->id, NULL, copy);
 		
 		if(!cr) {
 			print_message(("no such room: %s", copy));
@@ -1256,7 +1311,7 @@ static void process_commands(char *line)
 			msg = "Thanks, but no thanks!";
 		}
 		
-		cr = find_conf_room_by_name_and_id(ylad->id, room);
+		cr = find_conf_room_by_name_and_id(ylad->id, NULL, room);
 		if(!cr) {
 			print_message(("no such room: %s", room));
 			goto end_parse;
@@ -1540,13 +1595,31 @@ int main(int argc, char * argv[])
 	return 0;
 }
 
-void ext_yahoo_got_file(int id, char *who, char *url, long expires, char *msg, char *fname, unsigned long fesize)
+void ext_yahoo_got_file(int id, const char *me, const char *who, const char *url, long expires, const char *msg, const char *fname, unsigned long fesize)
 {
 }
 
 void ext_yahoo_got_identities(int id, YList * ids)
 {
 }
+void ext_yahoo_chat_yahoologout(int id, const char *me)
+{ 
+ 	LOG(("got chat logout for %s", me));
+}
+void ext_yahoo_chat_yahooerror(int id, const char *me)
+{ 
+ 	LOG(("got chat error for %s", me));
+}
+
+void ext_yahoo_got_ping(int id, const char *errormsg)
+{ 
+ 	LOG(("got ping errormsg %s", errormsg));
+}
+
+void ext_yahoo_got_search_result(int id, int found, int start, int total, YList *contacts)
+{
+	LOG(("got search result"));
+}
 
 static void register_callbacks()
 {
@@ -1570,6 +1643,8 @@ static void register_callbacks()
 	yc.ext_yahoo_chat_userjoin = ext_yahoo_chat_userjoin;
 	yc.ext_yahoo_chat_userleave = ext_yahoo_chat_userleave;
 	yc.ext_yahoo_chat_message = ext_yahoo_chat_message;
+	yc.ext_yahoo_chat_yahoologout = ext_yahoo_chat_yahoologout;
+	yc.ext_yahoo_chat_yahooerror = ext_yahoo_chat_yahooerror;
 	yc.ext_yahoo_got_webcam_image = ext_yahoo_got_webcam_image;
 	yc.ext_yahoo_webcam_invite = ext_yahoo_webcam_invite;
 	yc.ext_yahoo_webcam_invite_reply = ext_yahoo_webcam_invite_reply;
@@ -1582,6 +1657,7 @@ static void register_callbacks()
 	yc.ext_yahoo_typing_notify = ext_yahoo_typing_notify;
 	yc.ext_yahoo_game_notify = ext_yahoo_game_notify;
 	yc.ext_yahoo_mail_notify = ext_yahoo_mail_notify;
+	yc.ext_yahoo_got_search_result = ext_yahoo_got_search_result;
 	yc.ext_yahoo_system_message = ext_yahoo_system_message;
 	yc.ext_yahoo_error = ext_yahoo_error;
 	yc.ext_yahoo_log = ext_yahoo_log;
