@@ -1,5 +1,5 @@
 /***************************************************************************
-                          stuff.cpp  -  description
+                          interface_wireless_wirelessextensions.cpp  -  description
                              -------------------
     begin                : Sun May 6 2001
     copyright            : (C) 2001 by Stefan Winter
@@ -19,7 +19,7 @@
 #include <iwlib.h>
 #include <qdir.h>
 #include <qfile.h>
-#include <qstringlist.h>
+#include <qtable.h>
 
 #ifndef WITHOUT_ARTS
 #include <arts/artsflow.h>
@@ -57,12 +57,17 @@
 
 #define WIFI_GET_STATS(skfd, ifname, stats) \
 	iw_get_stats (skfd, ifname, stats, 0, 0)
-#else
+#define WIFI_PRINT_STATS(buff, event) \
+        iw_print_stats(buffer, sizeof(buffer), &event->u.qual, 0, 0);
+
+#else // HAVE_IW_25
 #define WIFI_CONFIG(config,field) config.field
-#define WIFI_GET_STATS(skfd, ifname, stats) \
-	iw_get_stats (skfd, ifname, stats)
 #define WIFI_EXTRACT_EVENT_STREAM(stream, event) \
 	iw_extract_event_stream(stream, event)
+#define WIFI_GET_STATS(skfd, ifname, stats) \
+	iw_get_stats (skfd, ifname, stats)
+#define WIFI_PRINT_STATS(buff, event) \
+        iw_print_stats(buffer, &event->u.qual, 0, 0);
 #endif
 
 /* ================================== FROM IWCONFIG.C ================================== */
@@ -303,24 +308,21 @@ QStringList Interface_wireless_wirelessextensions::available_wifi_devices()
 	  if (pos == -1)
 		continue;
 	  device = device.left(pos).stripWhiteSpace();
-	  if (!device.isEmpty())
-	    liste += device;
+	  if (device.isEmpty())
+		continue;
+	// Some drivers create two interfaces, ethX and wifiX.  The
+	// wifiX device are not useful to users because they only
+	// control the physical wlan settings, not the logical ones.  If
+	// we find a wifiX device then we move it to the start of the
+	// list which means the first instance of KWifiManger will pick
+	// up the ethX and only the second instance will pick up the
+	// wifiX.
+	  if (device.startsWith("wifi"))
+		liste.prepend(device);
+	  else
+		liste.append(device);	
   }
 
-  // Some drivers create two interfaces, ethX and wifiX.  The
-  // wifiX device are not useful to users because they only
-  // control the physical wlan settings, not the logical ones.  If
-  // we find a wifiX device then we move it to the start of the
-  // list which means the first instance of KWifiManger will pick
-  // up the ethX and only the second instance will pick up the
-  // wifiX.
-  for (QStringList::Iterator it = liste.begin (); it != liste.end (); ++it)
-  {
-	if ( (*it).contains("wifi") > 0 ) {
-		liste.remove(*it);
-		liste.prepend(*it);
-	}
-  }
 
   if (liste.isEmpty())
 	kdDebug () << "No wireless interface found." << endl;
@@ -421,7 +423,7 @@ bool Interface_wireless_wirelessextensions::poll_device_info ()
   essid = (WIFI_CONFIG(info,essid_on) ? WIFI_CONFIG(info,essid) : "any");
   char
     ap_addr[256];
-  iw_pr_ether (ap_addr, (const unsigned char *) info.ap_addr.sa_data);
+  iw_ether_ntop ( (const ether_addr *) info.ap_addr.sa_data, ap_addr);
   if (access_point_address != ap_addr)
     emitStatusChanged = true;
   access_point_address = ap_addr;
@@ -429,29 +431,42 @@ bool Interface_wireless_wirelessextensions::poll_device_info ()
     emitSpeedChanged = true;
   bitrate = info.bitrate.value;
   has_range = info.has_range;
-  if (has_range)
-    range = info.range.max_qual.qual;
+
   sigLevel[current] = tempic2.qual.level;
-  if (tempic2.qual.level < sigLevelMin)
-    sigLevelMin = tempic2.qual.level;
-  if (tempic2.qual.level > sigLevelMax)
-    sigLevelMax = tempic2.qual.level;
   noiseLevel[current] = tempic2.qual.noise;
-  if (tempic2.qual.noise < noiseLevelMin)
-    noiseLevelMin = tempic2.qual.noise;
-  if (tempic2.qual.noise > noiseLevelMax)
-    noiseLevelMax = tempic2.qual.noise;
+
+  if (has_range) {
+
+    range = info.range.max_qual.qual;
+  
+    // the following checks adjust values for sig levels that are in dBm
+    // according to iwlib.c. If offset is applied, the resulting
+    // value is negative and the GUI knows it can add the scale
+    // unit dBm. 
+
+    if (tempic2.qual.level > info.range.max_qual.level) {
+  	  sigLevel[current] = tempic2.qual.level - 0x100;
+          noiseLevel[current] = tempic2.qual.noise - 0x100;
+    }
+  }
+
+  if (sigLevel[current] < sigLevelMin)
+    sigLevelMin = sigLevel[current];
+  if (sigLevel[current] > sigLevelMax)
+    sigLevelMax = sigLevel[current];
+
+  if (noiseLevel[current] < noiseLevelMin)
+    noiseLevelMin = noiseLevel[current];
+  if (noiseLevel[current] > noiseLevelMax)
+    noiseLevelMax = noiseLevel[current];
   int
     tempqual = tempic2.qual.qual;
   if (has_range && (range != 0))
     tempqual = tempqual * 100 / range;
   if (      qual[current != 0 ? current - 1 : MAX_HISTORY-1] != tempqual)
     emitStrengthChanged = true;
-  if (  sigLevel[current != 0 ? current - 1 : MAX_HISTORY-1] != tempic2.qual.level)
-    emitStrengthChanged = true;
-  if (noiseLevel[current != 0 ? current - 1 : MAX_HISTORY-1] != tempic2.qual.noise)
-    emitStrengthChanged = true;
-  qual[current] = tempqual;
+
+  qual[current] = ( tempqual <= 100 ? tempqual : 100 );
   valid[current] = true;
 
   // try to get our local IP address
@@ -505,7 +520,6 @@ QString Interface_wireless_wirelessextensions::print_scanning_token (struct iw_e
   switch (event->cmd)
     {
     case SIOCGIWESSID:
-      {
 	char
 	  essid[IW_ESSID_MAX_SIZE + 1];
 	if ((event->u.essid.pointer) && (event->u.essid.length))
@@ -514,20 +528,49 @@ QString Interface_wireless_wirelessextensions::print_scanning_token (struct iw_e
 	if (event->u.essid.flags) {
 	  result = essid;
 	  }
-      }
-      break;
-    default:
       break;
 
-    }				/* switch(event->cmd) */
-  /* May have changed */
+    case SIOCGIWMODE:
+      if (event->u.mode == 1) { result = i18n("Ad-Hoc"); }
+      else if (event->u.mode == 3) { result = i18n("Managed"); }
+      else result = i18n("Unknown");
+      break;
+
+    case IWEVQUAL:
+      {
+	char buffer[128];
+	WIFI_PRINT_STATS(buffer, event);
+        result = buffer;
+        break;
+      }
+    case SIOCGIWENCODE:
+	if (event->u.data.flags & IW_ENCODE_DISABLED) {
+	  result = i18n( "off" );
+	} else {
+	  result = i18n( "on" );
+	}
+	break;
+   default:
+      break;
+
+  }                           /* switch(event->cmd) */
+
   return result;
 }
 
 
-QStringList Interface_wireless_wirelessextensions::get_available_networks ()
+QTable* Interface_wireless_wirelessextensions::get_available_networks ()
 {
-    QStringList	networks;
+
+    QTable* networks = new QTable(0,4,0);
+    networks->horizontalHeader()->setLabel( 0, i18n("Network Name") );
+    networks->horizontalHeader()->setLabel( 1, i18n("Mode") );
+    networks->horizontalHeader()->setLabel( 2, i18n("Quality") );
+    networks->horizontalHeader()->setLabel( 3, i18n("WEP") );
+
+    /* add 1, then: number of networks found */
+    int netcount = -1;
+
     struct iwreq wrq;
     unsigned char buffer[IW_SCAN_MAX_DATA];	/* Results */
     struct timeval tv;				/* Select timeout */
@@ -645,19 +688,47 @@ QStringList Interface_wireless_wirelessextensions::get_available_networks ()
 
 	    /* Currently, we only return the ESSIDs. We could
 	       collect further data, but that's something for
-	       maybe KDE 4.0 */
+	       maybe KDE 3.5 */
 
-	    if (ret > 0 && iwe.cmd == SIOCGIWESSID) {
-	      QString tempnet = print_scanning_token (&iwe);
-	      if (!tempnet.isEmpty()) 
-	        { networks += tempnet; }
-		else 
-		{ networks += (QString)i18n("(hidden network)"); }
+	    /* The first token is about ESSID. So if we see that, create a
+	       new row. If it's something else, it belongs to the network
+	       before. */
+
+	    if (ret > 0) {
+	      if (iwe.cmd == SIOCGIWESSID) {
+		 netcount++;
+		 kdDebug() << netcount << "\n";
+		 networks->insertRows(netcount);
+		 QString tempnet = print_scanning_token (&iwe);
+		 if (!tempnet.isEmpty()) 
+			{ networks->setText( netcount, 0, tempnet ); }
+			else 
+			{ networks->setText( netcount, 0, (QString)i18n("(hidden cell)")); }
+	      }
+	      if (iwe.cmd == SIOCGIWMODE) {
+                 QString tempresult= print_scanning_token (&iwe);
+                 networks->setText( netcount, 1, tempresult );
+		 
+	      }
+              if (iwe.cmd == IWEVQUAL) {
+                 QString tempresult= print_scanning_token (&iwe);
+		 // crop the result... it's a little too verbose
+		 // find everything between the first : and the first ' '
+		 tempresult = tempresult.mid(tempresult.find(':') + 1, tempresult.find( ' ', tempresult.find(':') ) - tempresult.find(':') );
+                 networks->setText( netcount, 2, tempresult );
+	      }
+	      if (iwe.cmd == SIOCGIWENCODE) {
+		   QString tempresult = print_scanning_token (&iwe);
+		   networks->setText( netcount, 3, tempresult ); 
+	      }
 	    }
+
 	  }
 	while (ret > 0);
       }
 
+    for (int i = 0; i<4; i++ ) networks->adjustColumn(i);
+
     return networks;
 }
 
